@@ -3,6 +3,10 @@
 #include        <string.h>
 #include	<ctype.h>
 #include	<stdio.h>
+#if defined(HAVE_LSTAT)
+#include	<sys/stat.h>
+#endif
+#include        <strstream>
 #include	"gr.hh"
 #include	"extern.hh"
 #include        "private.hh"
@@ -11,17 +15,20 @@
 #include        "command.hh"
 #include        "superus.hh"
 
-static vector<BlockSource> bsStack;
 
-void                   insert_cmd_in_ps(const char *cmd);
+std::vector<BlockSource> bsStack;
+
 static inline bool     testible(const char *s);
 static inline int      white_space(const char *sp);
 static inline int      word_length(const char *s);
-
+static bool            extract_help(FILE * fp, char *line);
+static bool            extract_procedure(FILE * fp, char *line);
+static bool            extract_syntax(char *line);
+static bool            perform_new_command(const char *s);
 
 
 // Store info about execution blocks.
-static bool     get_line_in_block(const char *block, unsigned *offset);
+bool             get_line_in_block(const char *block, unsigned int *offset);
 char           *_command_word_separator = "-----";
 int             _num_command_word = 0;
 char           *_command_word[MAX_cmd_word];
@@ -33,36 +40,28 @@ void            register_source(void);
 bool
 listCmd()
 {
-	int             i = 0, cmd, found = 0;
-	FILE           *fp;
-	string          tmpname_file;
 	if (_nword == 1) {
 		err("`list' what?");
 		return false;
 	}
 	// Try to use tempnam(), or tmpnam(), before using hardwired name 
-#if defined(HAVE_TEMPNAM)
-	tmpname_file.assign(tempnam("/usr/tmp", "gri"));
-#else
-#if defined(HAVE_TMPNAM)
-	tmpname_file.assign(tmpnam(NULL));
-#else
-	tmpname_file.assign(GRI_TMP_FILE);
-#endif
-#endif
+	FILE *fp;
+	std::string tmpname_file(tmp_file_name());
 	if (!(fp = fopen(tmpname_file.c_str(), "w"))) {
 		err("Error opening buffer-file for `list' command");
 		return false;
 	}
 	// Figure out what command, and give help for it. 
+	int i = 0;
 	while (!isspace(*(_cmdLine + i)))
 		i++;
 	while (isspace(*(_cmdLine + i)))
 		i++;
-	strcat(_cmdLine, "*");
-	for (cmd = 0; cmd < _num_command; cmd++) {
+	strcat(_cmdLine, " *");
+	bool found = false;
+	for (int cmd = 0; cmd < _num_command; cmd++) {
 		if (same_syntax(_cmdLine + i, _command[cmd].syntax, 1)) {
-			found = 1;
+			found = true;
 			fprintf(fp, "%s\n{\n", _command[cmd].help);
 			fprintf(fp, "%s}\n", _command[cmd].procedure);
 		}
@@ -121,10 +120,10 @@ display_cmd_being_done_stack()
 	int             i;
 	for (i = _cmd_being_done - 1; i > -1; i--) {
 		if (i == _cmd_being_done - 1)
-			gr_textput2("  called by:           `");
+			gr_textput("  called by:           `");
 		else
-			gr_textput2("  which was called by: `");
-		gr_textput2(_command[_cmd_being_done_code[i]].syntax);
+			gr_textput("  which was called by: `");
+		gr_textput(_command[_cmd_being_done_code[i]].syntax);
 		ShowStr("'\n");
 	}
 }
@@ -173,6 +172,16 @@ parse_C_commandCmd(const char *s)
 		}
 		// Scan to find command name.
 		i = 0;
+#if 0				// test name lengths
+		while (c_fcn[i].fcn) {
+			if (strlen(c_fcn[i].name)!= c_fcn[i].name_len) {
+				printf("code wants %d for \"%s\" but actually is %d\n", c_fcn[i].name_len,c_fcn[i].name,strlen(c_fcn[i].name));
+			}
+			i++;
+		}
+		exit(0);
+#endif
+
 		while (c_fcn[i].fcn) {
 			if (length == strlen(c_fcn[i].name)
 			    && !strncmp(s + start, c_fcn[i].name, length)) {
@@ -190,8 +199,12 @@ Gri cannot execute the command\n\
 		gr_Error(_grTempString);
 		return 0;		// will not be done
 	} else {
-		// It's not a C command.  It must be a gri command line, or series of
-		// lines.
+		// It's not a C command.  
+		// It must be a gri command line
+		// or a series of lines.
+#if 1
+		perform_new_command(s + start);
+#else
 		GET_STORAGE(_cmd_being_done_IP[_cmd_being_done],
 			    char, 1 + strlen(s + start));
 		strcpy(_cmd_being_done_IP[_cmd_being_done], s + start);
@@ -200,17 +213,58 @@ Gri cannot execute the command\n\
 			      _command[_cmd_being_done_code[_cmd_being_done - 1]].filename,
 			      _command[_cmd_being_done_code[_cmd_being_done - 1]].fileline
 			);
+
 		pop_command_word_buffer();
+#endif
 	}
 	return 1;			// BUG: no check on newcommands! 
 }				// parse_C_commandCmd() 
 #undef C_call
 #undef C_declaration
+
+bool
+perform_new_command(const char *s)
+{
+	GET_STORAGE(_cmd_being_done_IP[_cmd_being_done], char, 1 + strlen(s));
+	strcpy(_cmd_being_done_IP[_cmd_being_done], s);
+	push_command_word_buffer();
+	//printf("DEBUG %s:%d about to perform a NC <%s ...>\n",__FILE__,__LINE__,_cmdLine);
+	marker_draw();
+	perform_block(_cmd_being_done_IP[_cmd_being_done],
+		      _command[_cmd_being_done_code[_cmd_being_done - 1]].filename,
+		      _command[_cmd_being_done_code[_cmd_being_done - 1]].fileline
+		);
+	
+	marker_erase();
+	//printf("DEBUG %s:%d done performing NC\n",__FILE__,__LINE__);
+	pop_command_word_buffer();
+	free(_cmd_being_done_IP[_cmd_being_done]); // BUG: untested
+	return true;
+}
+
+void
+no_gri_cmd()
+{
+	std::string msg("ERROR: Gri can't locate the `gri.cmd' file\n");
+	msg.append("You need to tell Gri the directory containing this file.\n");
+	msg.append("  There are three ways in which you may do this:\n");
+	msg.append("    (1) Name the directory storing `gri.cmd' when you invoke Gri, e.g.\n");
+	msg.append("            gri -directory /usr/share/gri/lib\n");
+	msg.append("        where you may change the directory name to whatever directory\n");
+	msg.append("        contains the correct `gri.cmd' file, or ...\n");
+	msg.append("    (2) ... set an environment variable named GRI_DIRECTORY_LIBRARY\n");
+	msg.append("        to the name of a directory in which the `gri.cmd' file is\n");
+	msg.append("        located, or ...\n");
+	msg.append("    (3) ... recompile Gri so it will know where to look by default (see\n");
+	msg.append("        the INSTALL file in the source directory for instructions).\n");
+	gr_textput(msg.c_str());
+	gri_exit(1);
+}
 
 bool
-create_commands(const char *filename)
+create_commands(const char *filename, bool user_gave_directory)
 {
-	string fullfilename(_lib_directory.c_str());
+	std::string fullfilename(_lib_directory.c_str());
 	// Must check for '/' as file separator, on some machines.
 #if !defined(VMS)
 #if defined(MSDOS)
@@ -226,52 +280,48 @@ create_commands(const char *filename)
 #endif
 #endif
 	fullfilename += filename;
-	if (push_cmd_file(fullfilename.c_str(), false, false, "r")) {
-		/*
-		  First, see if the version number in gri.cmd matches hard-wired one.
-		  The gri.cmd is of the form:
-		  //         gri - scientific graphic program (version 2.1.6)
-		  and is contained on first line of file.
-		*/
-		get_command_line();
-		char *s = _cmdLine;
-		while (*s != '(')
-			s++;
-		int major_version, minor_version, minor_minor_version;
-		if (3 != sscanf(s, "(version %d.%d.%d)", 
-				&major_version, &minor_version, &minor_minor_version)) {
-			warning("Cannot find version info in gri.cmd");
-		} else {
-			int major2 = int(floor(1e-10+_version));
-			int minor2 = int(floor(100*(_version - major2)));
-			int minor_minor2 = int(floor(1e-10+10000*(_version - major2 - 0.01*minor2)));
-			if (major2 != major_version
-			    || minor2 != minor_version
-			    || minor_minor2 != minor_minor_version) {
-				char msg[200];
-				sprintf(msg, "Gri version number (%d.%d.%d) doesn't match version (%d.%d.%d) in library file gri.cmd", major2, minor2, minor_minor2, major_version, minor_version, minor_minor_version);
-				warning(msg);
-			}
-		}
-		// Finally, ok to process the startup file
-		while (do_command_line()) {
-			;			// EMPTY 
-		}
+	//printf("FIRST <%s>\n",fullfilename.c_str());
+	if (!push_cmd_file(fullfilename.c_str(), false, false, "r")) {
+		if (user_gave_directory)
+			no_gri_cmd(); // exits
+		char *gri_directory_library = egetenv("GRI_DIRECTORY_LIBRARY");
+		if (*gri_directory_library == '\0')
+			no_gri_cmd(); // exits
+		std::string envvar_location(gri_directory_library);
+		envvar_location.append("/gri.cmd");
+		//printf("TRY <%s>\n",envvar_location.c_str());
+		if (!push_cmd_file(envvar_location.c_str(), false, false, "r"))
+			no_gri_cmd(); // exits
+	}
+	/*
+	  First, see if the version number in gri.cmd matches hard-wired one.
+	  The gri.cmd is of the form:
+	  //         gri - scientific graphic program (version 2.1.6)
+	  and is contained on first line of file.
+	*/
+	get_command_line();
+	char *s = _cmdLine;
+	while (*s != '(')
+		s++;
+	int major_version, minor_version, minor_minor_version;
+	if (3 != sscanf(s, "(version %d.%d.%d)", 
+			&major_version, &minor_version, &minor_minor_version)) {
+		warning("Cannot find version info in gri.cmd");
 	} else {
-		// Couldn't find that file.
-		string msg("ERROR: Gri can't locate the startup file `");
-		msg += fullfilename;
-		msg += "'.\n\
-  You need to tell Gri the directory containing this file.\n\
-  There are several ways to do this:\n\
-    (1) Name the directory storing `gri.cmd' when you invoke Gri, e.g.\n\
-            gri -directory /opt/gri/lib\n\
-        where you may change the directory name to whatever directory\n\
-        contains the correct `gri.cmd' file.\n\
-    (2) Recompile Gri so it will know where to look by default (see\n\
-        the INSTALL file in the source directory for instructions).\n";
-		gr_textput(msg.c_str());
-		gri_exit(1);
+		int major2 = int(floor(1e-10+_version));
+		int minor2 = int(floor(1e-10 + 100.0*(_version - major2)));
+		int minor_minor2 = int(floor(1e-10+10000.0*(_version - major2 - 0.01*minor2)));
+		if (major2 != major_version
+		    || minor2 != minor_version
+		    || minor_minor2 != minor_minor_version) {
+			char msg[200];
+			sprintf(msg, "Gri version number (%d.%d.%d) doesn't match version (%d.%d.%d) in library file gri.cmd", major2, minor2, minor_minor2, major_version, minor_version, minor_minor_version);
+			warning(msg);
+		}
+	}
+	// Finally, ok to process the startup file
+	while (do_command_line()) {
+		;			// EMPTY 
 	}
 	return true;
 }
@@ -336,7 +386,7 @@ register_source()
 }
 
 // require closing ' to be on same line as opening ` 
-bool
+static bool
 extract_syntax(char *line)
 {
 	char  *cp;
@@ -382,12 +432,12 @@ bool extract_help(FILE * fp, char *line)
 		fgets(line, LineLength, fp);
 		_command[_num_command].fileline++;
 		_cmdFILE.back().increment_line();
-		insert_cmd_in_ps(line);
+		insert_cmd_in_ps(line/*, "command.cc:387"*/);
 		len = strlen(line);
 		size += len + 2;	// chars for NEWLINE and NULL (needed?) 
 		if (NULL == (cp = (char *) realloc(cp, size))) {
 			gr_Error("Can't reallocate space for help for new command");
-			free(cp);
+			free(cp); // BUG: need this?
 			return false;
 		}
 		// An unprotected '{' designates end of help text
@@ -449,19 +499,27 @@ extract_procedure(FILE * fp, char *line)
 #endif
 		if (feof(fp))
 			break;
-		insert_cmd_in_ps(line);
+		insert_cmd_in_ps(line /*, "command.cc:454"*/);
+		//printf("\n[%s] ", line);
+		remove_comment(line);
+		//printf("-> [%s]\n",line);
+		
 		// Test whether starting to define a newcommand within this one
 		if (*(line + skip_space(line)) == '`') {
 			err("Missing `}' in procedure body.");
 			return false;
 		}
-		if (*line == '}')
+		if (*line == '}') {
+			if (((unsigned) superuser()) & FLAG_AUT2) printf("%s:%d got procedure: <%s>\n",__FILE__,__LINE__,_command[_num_command].procedure);
 			return true;
+		}
 		_cmdFILE.back().increment_line();
 		unsigned len = 0;
 		while (line[len] != '\0' ) {
 			len++;
 		}
+		if (len == 0)
+			continue;
 		if (len > 1 && *(line + len - 2) == '\\')
 			*(line + len - 2) = '\0';
 		else if (*(line + len - 1) != '\n')
@@ -523,6 +581,8 @@ testible(const char *s)
 	if (*s == '[')  return false;
 	if (*s == '"')  return false;
 	if (*s == '{')  return false;
+	if (*s == '&')  return false;
+	if (*s == '*')  return false;
 	if (*s == '\n') return false;
 	if (*s == '\0') return false;
 	// May still be non-testible, if the next character after the next blank
@@ -591,8 +651,15 @@ perform_gri_cmd(int cmd)
 void
 push_command_word_buffer()
 {
-	int             i;
-	char            *cp = NULL;		// assignment prevents warning
+	// Figure out where last chunk started, so can nest & syntax,
+	int last_separator_at = -1;
+	for (int ii = _num_command_word - 1; ii >= 0; ii--) {
+		if (strEQ(_command_word[ii],_command_word_separator)) {
+			last_separator_at = ii;
+			break;
+		}
+	}
+	char *cp = NULL;		// assignment prevents warning
 	GET_STORAGE(cp, char, 1 + strlen(_command_word_separator));
 	strcpy(cp, _command_word_separator);
 	_command_word[_num_command_word] = cp;
@@ -600,10 +667,71 @@ push_command_word_buffer()
 	if (_num_command_word >= MAX_cmd_word) {
 		gr_Error("ran out of storage (must increase MAX_cmd_word in private.hh");
 	}
-	for (i = 0; i < _nword; i++) {
-		GET_STORAGE(cp, char, 1 + strlen(_word[i]));
-		strcpy(cp, _word[i]);
+	for (unsigned int i = 0; i < _nword; i++) {
+		//printf("DEBUG %s:%d push_command_word_buffer loop i= %d word[i]= <%s>\n",__FILE__,__LINE__,i,_word[i]);
+		if (*_word[i] == '&') {	// 2001-feb-10 trying new syntax
+			//printf("DEBUG %s:%d & found on word[%d] <%s>\n",__FILE__,__LINE__,i,_word[i]);
+			const char *name = 1 + _word[i];
+			char buf[300]; // BUG: should make bigger
+			int cmd_word_index = -1;
+			if (1 == sscanf(name, "\\.word%d.", &cmd_word_index)) {
+				// Nesting
+				//printf("DEBUG %s:%d & on \\.word%d.  last_sep at %d\n",__FILE__,__LINE__,cmd_word_index,last_separator_at);
+				std::string the_cmd_word;
+				//for (int ii = _num_command_word - 1; ii >= 0; ii--) printf("DEBUG %s:%d stack %3d [%s]  %d\n",__FILE__,__LINE__,ii,_command_word[ii],ii-last_separator_at);
+
+				if (last_separator_at + cmd_word_index + 1 < _num_command_word) {
+					char *cw = _command_word[last_separator_at + cmd_word_index + 1];
+					//printf("DEBUG %s:%d think it's [%s]\n",__FILE__,__LINE__,cw);
+					GET_STORAGE(cp, char, strlen(cw + 1));
+					strcpy(cp, cw);
+				} else {
+					err("Internal error command.cc:672; contact author");
+				}
+			} else if (is_var(name)) {
+				// Variable
+				int index = index_of_variable(name);
+				if (index < 0) {
+					err("cannot do \\", _word[i], " since `", name, "' does not exist", "\\");
+					return;
+				}
+				sprintf(buf, AMPERSAND_CODING, 1 + _word[i], marker_count());
+				GET_STORAGE(cp, char, 1 + strlen(buf));
+				strcpy(cp, buf);
+				//printf("DEBUG %s:%d WAS VAR.  made <%s>\n",__FILE__,__LINE__,cp);
+			} else if (is_syn(name)) {
+				// Synonym
+				int index = index_of_synonym(name);
+				if (index < 0) {
+					err("cannot do \\", _word[i], " since `", name, "' does not exist", "\\");
+					return;
+				}
+				sprintf(buf, AMPERSAND_CODING, 1 + _word[i], marker_count());
+				GET_STORAGE(cp, char, 1 + strlen(buf));
+				strcpy(cp, buf);
+				//printf("DEBUG %s:%d WAS SYN.  made <%s>\n",__FILE__,__LINE__,cp);
+			} else {
+				err("INTERNAL ERROR command.cc:677 -- notify author\n");
+				return;
+			}
+		} else if (is_var(_word[i])) { // 2000-dec-18 SF bug 122893
+			double v;
+			bool ok = get_var(_word[i], &v);
+			if (ok) {
+				char value[100];
+				sprintf(value, "%g", v);
+				GET_STORAGE(cp, char, 1 + strlen(value));
+				strcpy(cp, value);
+			} else {
+				GET_STORAGE(cp, char, 2);
+				strcpy(cp, "0");
+			}
+		} else {
+			GET_STORAGE(cp, char, 1 + strlen(_word[i]));
+			strcpy(cp, _word[i]);
+		}
 		_command_word[_num_command_word] = cp;
+		//printf("DEBUG %s:%d pushed command word %d as '%s'\n", __FILE__, __LINE__, _num_command_word, _command_word[_num_command_word]);
 		_num_command_word++;
 		if (_num_command_word >= MAX_cmd_word) {
 			gr_Error("ran out of storage (must increase MAX_cmd_word in private.hh");
@@ -614,11 +742,10 @@ push_command_word_buffer()
 void
 pop_command_word_buffer()
 {
-	int             i;
 	// Trash last end-point, then find new start-point by looking backwards
 	// for most recently pushed separator
 	if (_num_command_word > 1) {
-		for (i = _num_command_word - 1; i > -1; i--) {
+		for (int i = _num_command_word - 1; i > -1; i--) {
 			if (!strcmp(_command_word[i], _command_word_separator)) {
 				_num_command_word = i;
 				free((char *) _command_word[i]);
@@ -658,11 +785,14 @@ set_up_command_word_buffer()
 bool
 perform_block(const char *block, const char *source_file, int source_line)
 {
-	unsigned        lines = 0, offset = 0;
-	BlockSource bs(source_file, source_line);
+	unsigned int lines = 0, offset = 0;
+	BlockSource bs(block, source_file, source_line);
 	bsStack.push_back(bs);
 	// Scan through block line by line.  If a 'while' is found, then scan the
 	// loop and give it to perform_while_block.
+
+	extern int chars_read; // defined in read.cc
+
 	while (get_line_in_block(block, &offset)) {
 		lines++;
 		massage_command_line(_cmdLine);
@@ -673,22 +803,29 @@ perform_block(const char *block, const char *source_file, int source_line)
 		    && !skipping_through_if()) {
 			if (word_is(0, "quit"))
 				quitCmd();
+			bsStack.pop_back();
 			return false;
 		}
+		if (word_is(0, "read") && !skipping_through_if()) {
+			extern unsigned int offset_for_read;
+			offset_for_read = offset;
+		} else if (*_word[0] == '\\' && word_is(1, "=") && word_is(2, "system") && !skipping_through_if()) {
+			extern unsigned int offset_for_read;
+			offset_for_read = offset;
+		}
+
 		if (word_is(0, "while") && !skipping_through_if()) {
 			// Capture the loop (look for matching 'end while')
-			string          test;
-			char           *buffer;
-			unsigned        buffer_len = 1;
+			std::string          test;
 			unsigned        buffer_offset = offset;
 			int             loop_level = 1;
 			test.assign((char*)(6 + (char*)strstr(_cmdLine, "while")));
 			if (re_compare(test.c_str(), " *")) {
 				err("`while .test.|{rpn ...}' missing the test part");
+				bsStack.pop_back();
 				return false;
 			}
-			GET_STORAGE(buffer, char, 1);
-			*buffer = '\0';
+			std::string buffer;
 			while (get_line_in_block(block, &buffer_offset)) {
 				// Search for matching `end while'
 				if (re_compare(_cmdLine, "\\s*while.*")) {
@@ -699,49 +836,42 @@ perform_block(const char *block, const char *source_file, int source_line)
 						break;
 					}
 				}
-				buffer_len += 2 + strlen(_cmdLine);
-				buffer = (char *) realloc(buffer, buffer_len);
-				if (!buffer) {
-					gr_Error("ran out of storage during scanning of `while' loop");
-					return false;
-				}
-				strcat(buffer, _cmdLine);
-				strcat(buffer, "\n");
+				buffer.append(_cmdLine);
+				buffer.append("\n");
 			}
 			if (loop_level != 0) {
-				err("Missing `end while'");
+				err("Missing `end while'.");
+				bsStack.pop_back();
 				return false;
 			}
-			perform_while_block(buffer, test.c_str(), lines);
+			perform_while_block(buffer.c_str(), test.c_str(), lines);
 			// Adjust 'offset' to skip this interior loop, then skip the `end
 			// while' line.  Therefore will next capture line after the loop.
-			offset += strlen(buffer) + 1;	// point to after this loop 
+			offset += buffer.size() + 1;	// point to after this loop 
 			get_line_in_block(block, &offset);	// skip `end while' 
-			free(buffer);
+			//printf("AFTER THE WHILE, \n~~~%s~~~\n",block+offset);
 		} else if (word_is(0, "system")) {
-			// Intercept system commands, since if they are of the 
+			// Intercept system commands, since if they are of the
 			// <<EOF form, it will be neccessary to slurp the whole command
 			// right here.  Even if skipping through if, need
 			// to slurp until EOF
 			int status = 0;
 			char *s = _cmdLine;
-			char *cmd = NULL;		// assignment prevents warning
 			char *ptr;
 			s += skip_space(s);		// skip any initial space 
 			s += skip_nonspace(s);	// skip "system" 
 			s += skip_space(s);
-			GET_STORAGE(cmd, char, 1 + strlen(s));		
-			strcpy(cmd, s);
 			// s now points to first word after "system" 
 			if (*s == '\0' || *s == '\n') {
 				err("`system' needs a system command to do");
+				bsStack.pop_back();
 				return false;
 			}
 			ptr = (char*)strstr(s, "<<");
 			if (ptr) {
 				// It's of the form <<EOF or <<"EOF", so first find the
 				// stop word, named "read_until" here.
-				string read_until;
+				std::string read_until;
 				while(*ptr) {
 					if (*ptr != '<' && *ptr != '"' && !isspace(*ptr)) {
 						read_until.assign(ptr);
@@ -753,57 +883,54 @@ perform_block(const char *block, const char *source_file, int source_line)
 				int len = read_until.size();
 				for(int i = 0; i < len; i++) {
 					if (read_until[i] == '"' || isspace(read_until[i])) {
-						read_until[i] = '\0';
+						read_until.STRINGERASE(i, read_until.size() - i);
 						break;
 					}
 				}
+				static std::string cmd; // might save time in loops
+				cmd.assign(s);
 				while(get_line_in_block(block, &offset)) {
-					cmd = (char *) realloc(cmd, strlen(cmd) + 2 + strlen(_cmdLine));
-					if (!cmd) OUT_OF_MEMORY;
-					strcat(cmd, "\n");
-					strcat(cmd, _cmdLine);
-					if (len && !strncmp(_cmdLine + skip_space(_cmdLine), read_until.c_str(), len)) {
+					//printf("<%s> <%s> <%s>\n",_cmdLine, _cmdLine+skip_space(_cmdLine),read_until.c_str());
+					if (!strncmp(_cmdLine + skip_space(_cmdLine), read_until.c_str(), read_until.size())) {
+						cmd.append("\n");
+						cmd.append(_cmdLine + skip_space(_cmdLine));
+						//printf("PUT IN <%s>\nFULL IS:\n<%s>",_cmdLine + skip_space(_cmdLine),cmd.c_str());
 						break;
 					}
+					cmd.append("\n");
+					cmd.append(_cmdLine);
 				}
 				if (!skipping_through_if()) {
-					substitute_synonyms_cmdline(cmd, _cmdLine, false);
-					strcpy(cmd, _cmdLine);
-					if (_griState.superuser() & FLAG_SYS) {
-						ShowStr("\n`system' sending the following command to the operating system:\n");
-						ShowStr(cmd);
-						ShowStr("\n");
-					}
-					status = system(cmd);
+					std::string cmd_new;
+					substitute_synonyms_cmdline(cmd.c_str(), cmd_new, false);
+					status = call_the_OS(cmd_new.c_str(), __FILE__, __LINE__);
 					PUT_VAR("..exit_status..", (double) status);
 				}
-				free(cmd);
 			} else {
 				// It's just a simple system command. 
 				if (!skipping_through_if()) {
-					GET_STORAGE(cmd, char, 1 + strlen(s));
-					strcpy(cmd, s);
-					if (_griState.superuser() & FLAG_SYS) {
-						ShowStr("\n`system' sending the following command to the operating system:\n");
-						ShowStr(cmd);
-						ShowStr("\n");
-					}
-					status = system(cmd);
+					status = call_the_OS(s, __FILE__, __LINE__);
 					PUT_VAR("..exit_status..", (double) status);
-					free(cmd);
 				}
 			}
-		} else {
+		} else {	// It's not a system command
+			chars_read = 0;
 			perform_command_line(NULL, true);
+			offset += chars_read;
+			//printf("DEBUG %s:%d AFTER PERFORMING CMD LINE chars read= %d. next is what's left:\n{%s}\n",__FILE__,__LINE__,chars_read,block+offset);
 		}
+#if 0
 		// Increment offset 
-		bsStack[bsStack.size() - 1].incrementOffset();
+		bsStack[bsStack.size() - 1].increment_offset();
+#endif
 		// See if an error, or if `quit' executed 
 		stop_replay_if_error();
 		if (_done) {
+			bsStack.pop_back();
 			return false;
 		}
 	}				// while (get_line_in_block(block, &offset)) 
+	bsStack.pop_back();
 	return true;
 }				// perform_block
 
@@ -814,26 +941,23 @@ block_level()
 }
 
 // Do not call this if block_level()<=0 
-char
-*
+const char*
 block_source_file()
 {
 	if (bsStack.size() < 1) {
 		gr_Error("Underflow of block-stack (internal error)");
 	}
-	return bsStack[bsStack.size() - 1].getFilename();
+	return bsStack[bsStack.size() - 1].get_filename();
 }
 
 // Do not call this if block_level()<=0 
 unsigned int
 block_source_line()
 {
-	//printf("DEBUG:command.cc:block_source_line(): bsStack size %d\n",bsStack.size());
-	//printf("DEBUG:command.cc:block_source_line():  bsStack[bsStack.size() - 1].getLine()= %d\n", bsStack[bsStack.size() - 1].getLine());
 	if (bsStack.size() < 1) {
 		gr_Error("Underflow of block-stack (internal error)");
 	}
-	return bsStack[bsStack.size() - 1].getLine();
+	return bsStack[bsStack.size() - 1].get_line();
 }
 
 // Do not call this if block_level()<=0 
@@ -843,23 +967,27 @@ block_offset_line()
 	if (bsStack.size() < 1) {
 		gr_Error("Underflow of block-stack (internal error)");
 	}
-	return bsStack[bsStack.size() - 1].getOffset();
+	return bsStack[bsStack.size() - 1].get_offset();
 }
 
 // Return true if got a line 
 bool
-get_line_in_block(const char *block, unsigned *offset)
+get_line_in_block(const char *block, unsigned int *offset)
 {
-	int             i = 0;
+	//printf("DEBUG %s:%d in get_line_in_block ... <%s>\n",__FILE__,__LINE__,block+*offset);
+
 	if (*(block + *offset) == '\0')
 		return false;
+	unsigned int i = 0;
 	while (1) {
 		if (*(block + *offset) == '\n') {
 			*(_cmdLine + i) = '\0';
 			(*offset)++;
+			bsStack.back().increment_line(strlen(_cmdLine));
 			return true;
 		} else if (*(block + *offset) == '\0') {
 			*(_cmdLine + i) = '\0';
+			bsStack.back().increment_line(strlen(_cmdLine));
 			// Note -- next time will still point to '\0', and will catch 
 			return true;
 		}
