@@ -38,6 +38,7 @@ tracing()
 bool
 do_command_line()
 {
+	//printf("at start of do_command_line() _cmdLine is <%s>\n",_cmdLine);
 	// *** Get command line, storing it in the global _cmdLine ***
 	if (!get_command_line()) {
 		_cmdFILE.pop_back();
@@ -49,6 +50,7 @@ do_command_line()
 	}
 	// Remove comments, do math expansions, substitute synonyms, etc.
 	massage_command_line(_cmdLine);
+
 	// Handle "return" as special case
 	if ((_nword > 0)
 	    && (!strcmp(_word[0], "return"))
@@ -70,12 +72,21 @@ do_command_line()
 // Insert command line as a comment in the PostScript file, after
 // removing newpage character to blank, (since latex2e/epsfig breaks 
 // on newpage).
+// 
+// The 'note' is helpful in debugging.
 void
-insert_cmd_in_ps(const char *cmd)
+insert_cmd_in_ps(const char *cmd, const char *note)
 {
 	extern bool     _store_cmds_in_ps;	// DEFINED IN startup.c
 	if (!_store_cmds_in_ps)
 		return;
+	unsigned int first_nonwhite = 0;
+	while (isspace(*(cmd + first_nonwhite)))
+		first_nonwhite++;
+#if 0				// removed 2001-feb-22 for SF bug #133135
+	if (!strncmp(cmd + first_nonwhite, "insert", 6))
+		return;		// don't want 'insert' commands (confusing eh)
+#endif
 	strcpy(_grTempString, "gri:");
 	int ii = 4;			// where to start insert
 	int len = strlen(cmd);
@@ -88,6 +99,10 @@ insert_cmd_in_ps(const char *cmd)
 	_grTempString[ii] = '\0';
 	if (_grTempString[strlen(_grTempString) - 1] == '\n')
 		_grTempString[strlen(_grTempString) - 1] = '\0';
+	if (*note != '\0') {
+		strcat(_grTempString, " # ");
+		strcat(_grTempString, note);
+	}
 	gr_comment(_grTempString);
 }
 
@@ -100,12 +115,13 @@ insert_source_indicator(char *cl)
 	char line[1024];
 	sprintf(line, " \"%s:%d\"", _cmdFILE.back().get_name(), _cmdFILE.back().get_line());
 	strcat(cl, line);
-	if (cl[len - 1] == '\n')
+	if (len > 0 && cl[len - 1] == '\n')
 		cl[len - 1] = PASTE_CHAR;
 	else
 		cl[len] = PASTE_CHAR;
 }
 #endif
+
 // get_command_line() -- get a command line (skip full-line C comments)
 bool
 get_command_line(void)
@@ -114,15 +130,18 @@ get_command_line(void)
 	stop_replay_if_error();
 	// get a line from a file.
 	if (_cmdFILE.back().get_interactive()) {
+		//printf("DEBUG. interactive. before, had <%s>\n",_cmdLine);
 		// Cmd-file is interactive (from keyboard)
 		if (!gr_textget(_cmdLine, LineLength_1)) {
 			_done = 1;
 			return false;
 		}
+		//printf("DEBUG. interactive. after, have <%s>\n",_cmdLine);
 	} else {
 		// Cmd-file is non-interactive.
 		if (true == get_cmd(_cmdLine, LineLength_1, _cmdFILE.back().get_fp())) {
 			if (strlen(_cmdLine) < 1) {
+				if (((unsigned) superuser()) & FLAG_AUT2) printf("%s:%d debug 2\n",__FILE__,__LINE__);
 				return false;
 			} else {
 				warning("Missing newline at end of command file.");
@@ -141,16 +160,12 @@ get_command_line(void)
 		}
 	}
 	_cmdFILE.back().increment_line(); // BUG line numbers wrong BUG
-	insert_cmd_in_ps(_cmdLine);
-	if (!strncmp(_cmdLine + skip_space(_cmdLine), "/*", 2)) {
-		while (strncmp(_cmdLine + skip_space(_cmdLine), "*/", 2) && !_done) {
-			get_command_line();
-		}
-		get_command_line();
-	}
+	if (_cmdFILE.back().get_save2ps())
+		insert_cmd_in_ps(_cmdLine/*, "doline.cc:154"*/);
 	if (((unsigned) superuser()) & FLAG_AUT1) {
 		insert_source_indicator(_cmdLine);
 	}
+	if (((unsigned) superuser()) & FLAG_AUT2) printf("%s:%d  get_command_line returning [%s]\n",__FILE__,__LINE__,_cmdLine);
 	return true;
 }
 
@@ -193,41 +208,33 @@ remove_source_indicator(char *cmd)
 bool
 massage_command_line(char *cmd)
 {
+	_nword = 0;
 	if (((unsigned) superuser()) & FLAG_AUT1) {
 		remove_source_indicator(cmd);
 	}
 	remove_comment(cmd);
-	//printf("'%s %s'\n", cmd, is_system_command(cmd) ? "SYSTEM" : "NON_SYSTEM");
+
 	if (!is_system_command(cmd))
 		check_usage(cmd);
 	strcpy(_cmdLineCOPY, cmd + skip_space(cmd));
+
 	// For system commands, just substitute synonyms.
 	if (is_system_command(cmd)) {
 		strcpy(cmd, _cmdLineCOPY);
-		substitute_synonyms_cmdline(cmd, _cmdLineCOPY, false);
+		std::string cmd_sub;
+		substitute_synonyms_cmdline(cmd, cmd_sub, false);
+		strcpy(_cmdLineCOPY, cmd_sub.c_str());
 		strcpy(cmd, _cmdLineCOPY);
 		// Chop into words.  Note: chop_into_words() destroys it's string, so
 		// use a copy.
 		chop_into_words(_cmdLineCOPY, _word, &_nword, MAX_nword);
 		return true;
 	}
-	// For `\syn = ...' do not substitute to left of the "=".
-	if (*(cmd + skip_space(cmd)) == '\\') {
-		strcpy(_cmdLineCOPY, "");	// ensure null-terminated
 
-		substitute_synonyms_cmdline(cmd + skip_space(cmd), _cmdLineCOPY, false);
-		remove_trailing_blanks(_cmdLineCOPY);
-		strcpy(cmd, _cmdLineCOPY);
-		// Chop into words.  Note: chop_into_words() destroys it's string, so
-		// use a copy.
-		chop_into_words(_cmdLineCOPY, _word, &_nword, MAX_nword);
-		return true;
-	}
 	// Don't massage further if it's a `create new command' or a `postscript'
 	// command.
 	if (is_create_new_command(cmd)) {
-		if (((unsigned) superuser()) & FLAG_NEW)
-			printf("[%s]\n", cmd);
+		if (((unsigned) superuser()) & FLAG_NEW) printf("[%s]\n", cmd);
 		return true;
 	}
 	// Expand blanks, by separating words, but only in certain circumstances.
@@ -236,18 +243,20 @@ massage_command_line(char *cmd)
 	    && !re_compare(_cmdLine, "\\s*ls\\s*.*")
 	    && !re_compare(_cmdLine, "\\s*insert\\s*.*")
 	    && !re_compare(_cmdLine, "\\s*close\\s*.*")
-	    && !re_compare(_cmdLine, "\\s*open\\s*.*")
+	    && !re_compare(_cmdLine, "\\s*open\\s*.*") // removed 2.5.5; re-inserted 2.6.0
 	    && !re_compare(_cmdLine, "\\s*postscript\\s*.*") ) {
 		expand_blanks(cmd);
 	}
 	remove_trailing_blanks(cmd);
 	_error_in_cmd = false;
 	if (strlen(cmd) < 1) {
+		_nword = 0;
 		return true;
 	}
 	strcpy(_cmdLineCOPY, cmd + skip_space(cmd));
 	strcpy(cmd, _cmdLineCOPY);
 	if (strlen(cmd) < 1) {
+		_nword = 0;
 		return true;
 	}
 	// Copy back into cmd.  This must be done before substituting synonyms,
@@ -257,13 +266,13 @@ massage_command_line(char *cmd)
 		strcpy(cmd, _cmdLineCOPY);
 	}
 	// Substitute synonyms; copy back into cmd.
-	if (((unsigned) superuser()) & FLAG_SYN)
-		printf("[%s]\n", cmd);
+	if (((unsigned) superuser()) & FLAG_SYN) printf("[%s]\n", cmd);
 	// Don't substitute synonyms for these commands: `new \syn ...' `delete
 	// \syn ...'
 	if (re_compare(_cmdLine, "\\s*open\\s*.*")) {
-		substitute_synonyms_cmdline(cmd, _cmdLineCOPY, false);
-		strcpy(cmd, _cmdLineCOPY);
+		std::string cmd_sub;
+		substitute_synonyms_cmdline(cmd, cmd_sub, false);
+		strcpy(cmd, cmd_sub.c_str());
 	} else {
 		if (!re_compare(_cmdLine, "\\s*new\\s*.*")
 		    && !re_compare(_cmdLine, "\\s*delete\\s*.*")
@@ -274,13 +283,13 @@ massage_command_line(char *cmd)
 #endif
 		    && !re_compare(_cmdLine, "\\s*get\\s+env\\s*.*")
 			) {
-			substitute_synonyms_cmdline(cmd, _cmdLineCOPY, true);
-			strcpy(cmd, _cmdLineCOPY);
+			std::string cmd_sub;
+			substitute_synonyms_cmdline(cmd, cmd_sub, true);
+			strcpy(cmd, cmd_sub.c_str());
 			remove_trailing_blanks(cmd);
 		}
 	}
-	if (((unsigned) superuser()) & FLAG_SYN)
-		printf("  --> [%s]\n", cmd);
+	if (((unsigned) superuser()) & FLAG_SYN) printf("DEBUG %s:%d line is now '%s'\n",__FILE__,__LINE__,cmd);
 	// Substitute backtic and dollar-paren expressions
 #if USE_BACKTIC
 	sub_backtic(cmd, _cmdLineCOPY);
@@ -290,13 +299,14 @@ massage_command_line(char *cmd)
 	strcpy(cmd, _cmdLineCOPY);
 	// Substitute rpn expressions one by one, recopying back to cmd after
 	// each
-	if (((unsigned) superuser()) & FLAG_RPN)
-		printf("[%s]\n", cmd);
+	if (((unsigned) superuser()) & FLAG_RPN) printf("[%s]\n", cmd);
 	if (!re_compare(_cmdLine, "\\s*postscript\\s*.*")
 	    && !re_compare(_cmdLine, "\\s*new\\s*.*")
 	    && !re_compare(_cmdLine, "\\s*while\\s*.*") ) {
-		while (substitute_rpn_expressions(cmd, _cmdLineCOPY))
-			strcpy(cmd, _cmdLineCOPY);
+		if (((unsigned) superuser()) & FLAG_RPN) printf("DEBUG %s:%d about to substitute rpn in '%s'  skipping_through_if= %d\n",__FILE__,__LINE__,cmd,skipping_through_if());
+		if (!skipping_through_if())
+			while (substitute_rpn_expressions(cmd, _cmdLineCOPY))
+				strcpy(cmd, _cmdLineCOPY);
 		remove_trailing_blanks(cmd);
 		if (strlen(cmd) < 1)
 			return true;
@@ -304,8 +314,7 @@ massage_command_line(char *cmd)
 	}
 	stop_replay_if_error();
 	strcpy(cmd, _cmdLineCOPY);
-	if (((unsigned) superuser()) & FLAG_RPN)
-		printf("  --> [%s]\n", cmd);
+	if (((unsigned) superuser()) & FLAG_RPN) printf("  --> [%s]\n", cmd);
 	// Finally, chop up into words.  The words (_word) and number of words
 	// (_nword) are used all over the place in other functions.
 	chop_into_words(_cmdLineCOPY, _word, &_nword, MAX_nword);
@@ -317,7 +326,7 @@ massage_command_line(char *cmd)
 bool
 perform_command_line(FILE *fp, bool is_which)
 {
-	int             cmd;
+	//if (((unsigned) superuser()) & FLAG_RPN) printf("\nDEBUG %s:%d begin of perform_command_line\n",__FILE__,__LINE__);
 	if (strlen(_cmdLine) < 1)
 		return true;		// was ok, just blank
 	// If it's definition of a new gri command, do that.
@@ -333,20 +342,43 @@ perform_command_line(FILE *fp, bool is_which)
 		_done = 2;
 		return true;
 	}
+
+
+	if (((unsigned) superuser()) & FLAG_RPN) printf("DEBUG %s:%d perform_command_line() has command '%s'\n", __FILE__, __LINE__, _cmdLine);
+	if (((unsigned) superuser()) & FLAG_RPN) printf("DEBUG %s:%d skipping_through_if is %d (STEP 1)\n", __FILE__, __LINE__, skipping_through_if());
+
+
 	// Handle `end' and `else'
-	if (handle_if_block())
+	if (handle_if_block()) {
+		if (((unsigned) superuser()) & FLAG_RPN) printf("DEBUG: %s:%d returning early\n",__FILE__,__LINE__);
 		return true;
+	}
+
+	if (((unsigned) superuser()) & FLAG_RPN) printf("DEBUG %s:%d skipping_through_if is %d (STEP 2)\n", __FILE__, __LINE__, skipping_through_if());
+
 	// Process line if not skipping
 	if (!skipping_through_if()) {
+		// First, handle de-referenced synonyms as lvalues (to left
+		// an assignment operator)
+		std::string w0(_word[0]);
+                // de_reference(w0); BUG: MAY PUT BACK LATER
+		char *cp = strdup(w0.c_str());
+		_word[0] = cp;
 		// Handle `\name = "value"' command
-		if (*_word[0] == '\\') {
-			if (_nword >= 3 && !strcmp(_word[1], "=")) {
+		if (w0[0] == '\\') {
+			if (_nword > 2 && is_assignment_op(_word[1])) {
 				assign_synonym();
+				free(cp);
 				return true;
 			}
 		}
-		// Handle math command specially. Examples: `x += 1', `grid data +=
-		// 1', `image += 1', `.var. += 1'
+		// Handle math command, e.g.
+		//     .var. = 1
+		//     x += 1
+		//     grid data += 1
+		//     image += 1
+		// etc., permitting various assignment operators,
+		// including '=', '+=', etc.
 		if (word_is(0, "x")
 		    || word_is(0, "y")
 		    || word_is(0, "z")
@@ -354,7 +386,7 @@ perform_command_line(FILE *fp, bool is_which)
 		    || word_is(0, "v")
 		    || word_is(0, "image")
 		    || word_is(0, "grid")
-		    || is_var(_word[0])) {
+		    || is_var(w0)) {
 			if (_nword == 3) {
 				if(word_is(1, "=") 
 				   || word_is(1, "-=")
@@ -363,7 +395,9 @@ perform_command_line(FILE *fp, bool is_which)
 				   || word_is(1, "/=")
 				   || word_is(1, "^=")
 				   || word_is(1, "_=")) {
+					//printf("DOING mathCmd\n"); for (int ii=0;ii<_nword;ii++) printf("\t<%s>\n",_word[ii]);
 					mathCmd();
+					free(cp);
 					return true;
 				}
 			} else if (_nword == 4 
@@ -371,6 +405,7 @@ perform_command_line(FILE *fp, bool is_which)
 							     || word_is(1, "x")
 							     || word_is(1, "y")) ) {
 				mathCmd();
+				free(cp);
 				return true;
 			} else if (_nword == 4
 				   && word_is(0, "image") && (word_is(1, "grayscale")
@@ -378,39 +413,49 @@ perform_command_line(FILE *fp, bool is_which)
 							      || word_is(1, "colorscale")
 							      || word_is(1, "colourscale"))) {
 				mathCmd();
+				free(cp);
 				return true;
 			} else {
 				err("Unknown command.  Were you trying to manipulate `\\",
 				    _word[0], "' mathematically?", "\\");
+				free(cp);
 				return false;
 			}
 		}
 		// Figure out what command, and do it.
+		int cmd;
 		if (0 != (cmd = match_gri_syntax(_cmdLine, 0))) {
 			// Do the command.
 			push_cmd_being_done_stack(cmd - 1);
 			if (!perform_gri_cmd(cmd - 1)) {
 				err("Can't perform/parse following command");
+				free(cp);
 				return false;
 			}
 			pop_cmd_being_done_stack();
+			free(cp);
 			return true;
 		} else {
 			// No syntax registered for this command.  Check special cases,
 			// where it's a stray "break," or "continue"
 			if (word_is(0, "break")) {
 				err("Cannot have `break' outside loops");
+				free(cp);
 				return false;
 			} else if (word_is(0, "continue")) {
 				err("Cannot have `continue' outside loops");
+				free(cp);
 				return false;
-			} else if(string_is_blank(_word[0])) {
+			} else if (string_is_blank(_word[0])) {
+				free(cp);
 				return true;
 			} else {
 				err("Unknown command encountered.");
+				free(cp);
 				return false;
 			}
 		}
+		free(cp);
 	}
 	return true;
 }
@@ -439,7 +484,7 @@ is_system_command(const char *s)
 	if (!strncmp(s, "system", 6) ? true : false)
 		return true;
 
-    // Second, check if '\s*\\name\s*=\s*system ...'
+	// Second, check if '\s*\\name\s*=\s*system ...'
 	s += skip_nonspace(s);      // name
 	s += skip_space(s);         // space
 	s += skip_nonspace(s);      // =
@@ -469,13 +514,11 @@ is_system_command(const char *s)
 bool
 systemCmd()
 {
-	int             status = 0;
-	char           *s = _cmdLine;
-	int             i, icmd, len;
 	if (skipping_through_if())
 		return true;
 	// Much of following code duplicated in assign_synonym(), so if any
 	// problems crop up, check there too.
+	char *s = _cmdLine;
 	s += skip_space(s);		// skip any initial space
 	s += skip_nonspace(s);	// skip "system"
 	s += skip_space(s);
@@ -486,14 +529,17 @@ systemCmd()
 	}
 	// See if last word starts with "<<"; if so, then the stuff to be done
 	// appears on the lines following, ended by whatever word follows the
-	// "<<".
-	i = strlen(s) - 2;
-	GriString read_until;
+	// "<<". 
+	// ... compare set.cc near line 3288.
+	int i = strlen(s) - 2;
+	static std::string read_until;
+	read_until.assign("");
 	bool            using_read_until = false;
 	while (--i) {
 		if (!strncmp((s + i), "<<", 2)) {
+			//printf("- looking for <<\n");
 			bool            quoted_end_string = false;
-			int             ii, spaces = 0;
+			int             spaces = 0;
 			while (isspace(*(s + i + 2 + spaces))) {
 				spaces++;
 			}
@@ -501,30 +547,33 @@ systemCmd()
 				spaces++;
 				quoted_end_string = true;
 			}
-			read_until.fromSTR(s + i + 2 + spaces);
+			read_until.assign(s + i + 2 + spaces);
 			using_read_until = true;
-			// trim any trailing whitespace from search word
-			ii = 0;
-			while (read_until[ii]) {
-				if (isspace(read_until[ii])
-				    || (quoted_end_string && '"' == read_until[ii])) {
-					read_until[ii] = '\0';
-					break;
-				}
-				ii++;
-			}
+			// trim junk from end of the 'read until' string
+			std::string::size_type cut_at;
+			if (quoted_end_string)
+				cut_at = read_until.find("\"");
+			else
+				cut_at = read_until.find(" ");
+			//printf("READING UNTIL '%s' ... i.e.\n", read_until.c_str());
+			if (cut_at != STRING_NPOS)
+				read_until.STRINGERASE(cut_at, read_until.size() - cut_at);
 			if (read_until.size() < 1) {
 				err("`system ... <<STOP_STRING' found no STOP_STRING");
 				return false;
 			}
+			//printf("reading until '%s'\n",read_until.c_str());
 			break;
 		}
 	}
-	GriString             cmd(s);
+	static std::string cmd;	// might save time in loops
+	cmd.assign(s);
 	if (using_read_until) {
 		// It is of the <<WORD form
-		cmd.catSTR("\n");
+		cmd.append("\n");
+		//printf("%s:%d ABOUT TO GOBBLE\n",__FILE__,__LINE__);
 		while (get_command_line()) {
+			//printf("%s:%d cmd line is [%s]\n",__FILE__,__LINE__,_cmdLine);
 			// Trim filename/fileline indicator
 			unsigned int l = strlen(_cmdLine);
 			for (unsigned int ii = 0; ii < l; ii++) {
@@ -533,44 +582,39 @@ systemCmd()
 					break;
 				}
 			}
-			cmd.catSTR(_cmdLine);
-			cmd.catSTR("\n");
-			if (!strncmp(_cmdLine, read_until.getValue(), read_until.size())) {
+			if (!strncmp(_cmdLine + skip_space(_cmdLine), read_until.c_str(), read_until.size())) {
+				cmd.append(_cmdLine + skip_space(_cmdLine));
+				cmd.append("\n");
 				break;
 			}
+			cmd.append(_cmdLine);
+			cmd.append("\n");
 		}
-		substitute_synonyms_cmdline(cmd.getValue(), _cmdLine, false);
-		cmd.fromSTR(_cmdLine);
+		static std::string cmd_sub;
+		substitute_synonyms_cmdline(cmd.c_str(), cmd_sub, false);
+		cmd = cmd_sub;
 	} else {
 		// No, it is not of the <<WORD form
-		len = cmd.size();
-		char *ptr = cmd.getValue();
-		for (icmd = 0, i = 0; i <= len; icmd++, i++) {
-			if (i > 2
-			    && s[i] == 'n'
-			    && s[i - 1] == '\\'
-			    && s[i - 2] == '\\') {
-				icmd--;		// overwrite so \\n becomes \n
-			}
-			*(ptr + icmd) = *(s + i);
+		std::string::size_type loc = 0;
+		//printf("system command BEFORE [%s]\n",cmd.c_str());
+		while (STRING_NPOS != (loc = cmd.find("\\\\", loc))) {
+			cmd.STRINGERASE(loc, 2);
+			cmd.insert(loc, "\\");
 		}
+		//printf("AFTER [%s]\n",cmd.c_str());
 	}
-	if (((unsigned) superuser()) & FLAG_SYS) {
-		ShowStr("\n`system' sending the following command to the operating system:\n");
-		ShowStr(cmd.getValue());
-		ShowStr("\n");
-	}
-	status = system(cmd.getValue());
+	int status = call_the_OS(cmd.c_str(), __FILE__, __LINE__);
 	PUT_VAR("..exit_status..", (double) status);
 	sprintf(_grTempString, "%d status\n", status);
 	RETURN_VALUE(_grTempString);
+
 	return true;
 }
 
 static void
 show_startup_msg()
 {
-	string fullfilename(_lib_directory.c_str());
+	std::string fullfilename(_lib_directory.c_str());
 	// Must check for '/' as file separator, on some machines.
 #if !defined(VMS)
 #if defined(MSDOS)
@@ -638,32 +682,65 @@ write_prompt()
 // remove_comment() -- fix command line 1) check that input line read OK 2)
 // remove the '\n' from fgets 3) remove trailing comments 4) remove trailing
 // blanks or tabs
-void
+//
+// Return true if there WAS a comment and it was the FIRST non-whitespace thing.
+bool
 remove_comment(char *s)
 {
-	register char  *sPtr;
-	register int    i, len;
-	int             inquote = 0;
-
+	//printf("\tremove_comment(%s)\n",s);
+	int len = strlen(s);
+#if 0
 	// Discard trailing newlines (or control-M characters, which
 	// are added to newline in PC-ish systems), by converting
 	// them to nulls
-	len = strlen(s);
-	while (s[len - 1] == '\n' || s[len - 1] == '\r')
+	while (len > 1 && (s[len - 1] == '\n' || s[len - 1] == '\r'))
 		s[--len] = '\0';
-
+#endif
+	bool is_continued = false;
+	if (len > 1 && s[len - 1] == '\\' && s[len - 2] == '\\') 
+		is_continued = true;
 	// Discard comment on end
 	if (len > 0) {
-		for (sPtr = s, i = 0; i < len; i++, sPtr++) {
-			if (*sPtr == '"')
-				inquote = !inquote;
-			else if (!inquote && (!strncmp(sPtr, "//", 2) || *sPtr == '#')) {
-				*sPtr = '\0';
+		bool in_dquote = false;
+		bool in_squote = false;
+		for (int i = 0; i < len; i++) {
+			if (s[i] == '"') {
+				if (!in_squote)
+					in_dquote = !in_dquote;
+				continue;
+			}
+			if (s[i] == '\'') {
+				if (!in_dquote)
+					in_squote = !in_squote;
+				continue;
+			}
+			if (!in_dquote
+			    && !in_squote
+			    && ((s[i] == '/' && s[i+1] == '/') || s[i] == '#')) {
+				//printf("remove_comment [%s] ->\n", s);
+				s[i] = '\0';
+				//printf("               [%s]\n", s);
+				if (is_continued) {
+					s[i++] = '\\';
+					s[i++] = '\\';
+				}
 				len = i;
 				break;
 			}
 		}
+		// Set a flag if there is now nothing but whitespace.
+		for (int ii = 0; ii < len; ii++) {
+			//printf("\tEXAMINE [%c]\n", s[ii]);
+			if (!isspace(s[ii])) {
+				//printf("\t\tRETURNING false [%s]\n", s);
+				return false;
+			}
+		}
+		//printf("\t\tRETURNING true [%s]\n", s);
+		return true;
 	}
+	//printf("\t\tRETURNING BOTTOM false [%s]\n", s);
+	return false;
 }
 
 static void
@@ -671,6 +748,7 @@ check_usage(const char *s)
 {
 	if (s[0] == '`')
 		return;			// defining new command
+#if 0				// 2.5.5 allows e.g \.argv[0].
 	register int    i, len;
 	int             inquote = 0;
 	len = strlen(s);
@@ -696,6 +774,7 @@ except inside system calls\n\
 			ShowStr(_grTempString);	    
 		}
 	}
+#endif
 }
 
 #if USE_BACKTIC
@@ -787,8 +866,8 @@ sub_dollar_paren(const char *in, char *out)
 	GriString in_copy(in);
 	bool inserted_something = false;
 	while (1) {
-		vector<int> start;	// where \$( sequences start
-		vector<int> depth;	// depth of these sequences
+		std::vector<int> start;	// where \$( sequences start
+		std::vector<int> depth;	// depth of these sequences
 		int level = 0, max_level = 0;
 		unsigned int i;
 		for (i = 3; i < len; i++) {
@@ -995,6 +1074,7 @@ get_cmd(char *buf, int max, FILE *fp)
 		// Check for EOF 
 		if (thisc == EOF) {
 			*(buf + i) = '\0';
+			if (((unsigned) superuser()) & FLAG_AUT2) printf("%s:%d get_cmd hit EOF, now returning TRUE with [%s]\n",__FILE__,__LINE__,buf);
 			return true;
 		}
 		// See if NEWLINE  (even if quoted) 
@@ -1006,6 +1086,7 @@ get_cmd(char *buf, int max, FILE *fp)
 				continue;
 			} else {
 				*(buf + i) = '\0';
+				if (((unsigned) superuser()) & FLAG_AUT2) printf("%s:%d get_cmd IN MIDDLE returning FALSE with [%s]\n",__FILE__,__LINE__,buf);
 				return false;
 			}
 		}
@@ -1014,5 +1095,6 @@ get_cmd(char *buf, int max, FILE *fp)
 		lastc = thisc;
 	} while (i++ < max);
 	*(buf + i - 1) = '\0';	// didn't get to end - bad 
+	printf("%s:%d get_cmd AT END returning FALSE with [%s]\n",__FILE__,__LINE__,buf);
 	return false;
 }
