@@ -195,8 +195,17 @@ draw_symbolCmd()
 	bool            hue_in_z = false, saturation_in_z = false, brightness_in_z = false;
 	double          hue = 1.0, saturation = 1.0, brightness = 1.0;
 	gr_symbol_type  symbolCode = gr_bullet_symbol; // will be changed
-	set_x_scale();
-	set_y_scale();
+
+	if (!scales_defined()) {
+#if 0				// Fix SF bug #129856 (I hope!)
+		no_scales_error();
+		return false;
+#else
+		create_x_scale();
+		create_y_scale();
+#endif
+	}
+
 	// Columns must exist, if not the "at" style, which is for a single point
 	if (!word_is(3, "at")) {
 		if (!_columns_exist) {
@@ -207,13 +216,18 @@ draw_symbolCmd()
 	// Scales must be defined unless symbol location given in cm
 	if (!word_is(_nword - 1, "cm")) {
 		if (!scales_defined()) {
+#if 0				// Fix SF bug #129856 (I hope!)
 			no_scales_error();
 			return false;
+#else
+			create_x_scale();
+			create_y_scale();
+#endif
 		}
 	}
 	// Is it brightness in z?
 	bool uses_graylevel = false;
-	if (word_is(2, "graylevel")) {
+	if (word_is(2, "graylevel")) { // draw symbol graylevel z
 		if (!word_is(3, "z")) {
 			err("Word following 'graylevel' must be 'z'");
 			demonstrate_command_usage();
@@ -225,8 +239,7 @@ draw_symbolCmd()
 			return false;
 		}
 		uses_graylevel = true;
-	}
-	if (word_is(3, "graylevel")) {
+	} else if (word_is(3, "graylevel")) { // draw symbol NAME graylevel z
 		if (!word_is(4, "z")) {
 			err("Word following 'graylevel' must be 'z'");
 			demonstrate_command_usage();
@@ -238,13 +251,17 @@ draw_symbolCmd()
 			return false;
 		}
 		uses_graylevel = true;
+		if (gr_unknown_symbol == (symbolCode = determine_symbol_code(_word[2]))) {
+			demonstrate_command_usage();
+			err("Can't understand symbol \\`", _word[2], "'", "\\");
+			return false;
+		}
+		fixedSymbol = true;
 	}
 	// If it's color, extract hue/saturation/brightness
 	if (!uses_graylevel) {
-		if (word_is(2, "color") || word_is(2, "colour")
-		    || word_is(3, "color") || word_is(3, "colour")) {
+		if (word_is(2, "color") || word_is(2, "colour") || word_is(3, "color") || word_is(3, "colour")) {
 			bool            OLD = _ignore_error;
-			int             i;
 			_ignore_error = true;	// can't read "z" in "hue z" as number
 			if (1 == get_cmd_values(_word, _nword, "saturation", 1, _dstack))
 				saturation = _dstack[0];
@@ -253,7 +270,7 @@ draw_symbolCmd()
 			if (1 == get_cmd_values(_word, _nword, "hue", 1, _dstack))
 				hue = _dstack[0];
 			// Figure if anything coded into z column
-			for (i = 2; i < _nword; i++) {
+			for (unsigned int i = 2; i < _nword; i++) {
 				if (word_is(i, "z")) {
 					if (word_is(i - 1, "brightness"))
 						brightness_in_z = true;
@@ -334,7 +351,10 @@ draw_symbolCmd()
 					set_ps_color('p');
 					set_line_width_symbol();
 					gr_drawsymbol(xcm, ycm, symbolCode);
+					PUT_VAR("..xlast..", x);
+					PUT_VAR("..ylast..", y);
 				}
+				draw_axes_if_needed();
 				return true;
 			case 7:
 				// `draw symbol .code.|\name at .x. .y. cm' 
@@ -377,6 +397,8 @@ draw_symbolCmd()
 					set_ps_color('p');
 					set_line_width_symbol();
 					gr_drawsymbol(x, y, symbolCode);
+					PUT_VAR("..xlast..", x);
+					PUT_VAR("..ylast..", y);
 				}
 				return true;
 			default:
@@ -397,6 +419,8 @@ draw_symbolCmd()
 	int num = _colX.size();
 	GriColor c, old_color = _griState.color_line();
 	set_line_width_symbol();
+	double xlast = gr_currentmissingvalue();
+	double ylast = gr_currentmissingvalue();
 	for (i = 0; i < num; i++) {
 		if (!gr_missingx((double) *xp)
 		    && !gr_missingy((double) *yp)
@@ -423,10 +447,14 @@ draw_symbolCmd()
 					set_ps_color('p');
 				}
 				gr_drawsymbol(xcm, ycm, symbolCode);
+				xlast = *xp;
+				ylast = *yp;
 			} else {
 				// Symbol stored in z 
 				if (!gr_missing((double) *zp)) {
 					gr_drawsymbol(xcm, ycm, gr_symbol_type(int(floor((0.5 + *zp)))));
+					xlast = *xp;
+					ylast = *yp;
 				}
 			}
 		}
@@ -434,7 +462,6 @@ draw_symbolCmd()
 		yp++;
 		zp++;
 	}
-	_drawingstarted = true;
 	if (uses_color || uses_graylevel) {
 		_griState.set_color_line(old_color);
 		set_ps_color('p');
@@ -445,11 +472,13 @@ draw_symbolCmd()
 			sprintf(_grTempString, "\
 `draw symbol' drew %d of the %d data; the remaining %d points\n\
   were either missing or out of clip region.\n",
-				num_drawn, _colX.size(), _colX.size() - num_drawn);
+				num_drawn, int(_colX.size()), int(_colX.size()) - num_drawn);
 			gr_textput(_grTempString);
 		}
 
 	}
+	PUT_VAR("..xlast..", xlast);
+	PUT_VAR("..ylast..", ylast);
 	return true;
 }
 
@@ -546,7 +575,7 @@ draw_contourCmd()
 	double          contour_minlength = _contour_minlength;
 	double          xsize = XSIZE_DEFAULT;
 	double          ysize = YSIZE_DEFAULT;
-	string          user_label;
+	std::string     user_label;
 	int             nword = _nword;
 	bool            user_gave_label = false;	// only for .value. format
 	bool            have_unlabelled_and_labelled = false;
@@ -556,8 +585,13 @@ draw_contourCmd()
 	if (!grid_exists())
 		return false;
 	if (!scales_defined()) {
+#if 0				// Fix SF bug #129856 (I hope!)
 		no_scales_error();
 		return false;
+#else
+		create_x_scale();
+		create_y_scale();
+#endif
 	}
 	// Decode command
 	labelled = true;
@@ -588,10 +622,7 @@ draw_contourCmd()
 			return false;
 		user_gave_label = true;	// label is _word[4]
 		user_label.assign(_word[4]);
-		if (user_label[0] == '"')
-			user_label.STRINGERASE(0, 1);
-		if (user_label[user_label.size()-1] == '"')
-			user_label.STRINGERASE(user_label.size()-1,1);
+		un_double_quote(user_label);
 		dmin = level;
 		dmax = level;
 		dinc = level;
@@ -702,7 +733,7 @@ draw_contourCmd()
 			gr_textput(_grTempString);
 		}
 		if (labelled
-		    && (numcontours == 1 || gr_multiple(dlevel, dinc, 0.001 * dinc))) {
+		    && (numcontours == 1 || gr_multiple(dmax - dlevel, dinc, 0.001 * dinc))) {
 			if (fabs(dlevel) <= 1.0e-6 * fabs(dinc)) {
 				label.fromSTR("0");
 			} else {
@@ -801,8 +832,13 @@ draw_arrow_from_toCmd()
 	if (strcmp(_word[_nword - 1], "cm")) {
 		double          x_cm, y_cm;
 		if (!scales_defined()) {
+#if 0				// Fix SF bug #129856 (I hope!)
 			no_scales_error();
 			return false;
+#else
+			create_x_scale();
+			create_y_scale();
+#endif
 		}
 		gr_usertocm(x0, y0, &x_cm, &y_cm);
 		x0 = x_cm;
@@ -842,8 +878,13 @@ draw_arrowsCmd()
 		return true;
 	}
 	if (!scales_defined()) {
+#if 0				// Fix SF bug #129856 (I hope!)
 		no_scales_error();
 		return false;
+#else
+		create_x_scale();
+		create_y_scale();
+#endif
 	}
 	if (!_uscale_exists) {
 		demonstrate_command_usage();
@@ -962,8 +1003,13 @@ draw_axes(int type, double loc, gr_axis_properties side, bool allow_offset)
 	double          tmpx, tmpx_cm, tmpy, tmpy_cm;
 	double          axes_offset;
 	if (!scales_defined()) {
+#if 0				// Fix SF bug #129856 (I hope!)
 		no_scales_error();
 		return false;
+#else
+		create_x_scale();
+		create_y_scale();
+#endif
 	}
 	if (allow_offset)
 		axes_offset = _axes_offset;
@@ -999,11 +1045,11 @@ draw_axes(int type, double loc, gr_axis_properties side, bool allow_offset)
 	set_y_scale();
 	// Set to proper linewidth, and turn dashing off
 	set_line_width_axis();
-	vector<double> old_dash;
+	std::vector<double> old_dash;
 	unsigned int i;
 	for (i = 0; i < _dash.size(); i++)
 		old_dash.push_back(_dash[i]);
-	vector<double> dash;
+	std::vector<double> dash;
 	_dash.erase(_dash.begin(), _dash.end());
 	switch (type) {
 	case 0:			// full axes 
@@ -1133,8 +1179,13 @@ draw_curveCmd()
 		return true;
 	}
 	if (!scales_defined()) {
+#if 0				// Fix SF bug #129856 (I hope!)
 		no_scales_error();
 		return false;
+#else
+		create_x_scale();
+		create_y_scale();
+#endif
 	}
 	bool filled = false;
 	if (_nword == 6
@@ -1199,8 +1250,8 @@ draw_curveCmd()
 		path.fill(units_user);
 	else
 		path.stroke(units_user);
-	PUT_VAR("..lastx..", lastx);
-	PUT_VAR("..lasty..", lasty);
+	PUT_VAR("..xlast..", lastx);
+	PUT_VAR("..ylast..", lasty);
 	_drawingstarted = true;
 	draw_axes_if_needed();
 	return true;
@@ -1221,8 +1272,13 @@ draw_curve_filled_to_valueCmd(bool to_y, double value)
 		return true;
 	}
 	if (!scales_defined()) {
+#if 0				// Fix SF bug #129856 (I hope!)
 		no_scales_error();
 		return false;
+#else
+		create_x_scale();
+		create_y_scale();
+#endif
 	}
 	GriPath p;
 	set_environment();
@@ -1301,8 +1357,8 @@ draw_gridCmd()
 	}
 	set_environment();
 	set_line_width_symbol();
+	set_ps_color('p');
 	unsigned j = _num_ymatrix_data - 1;
-	set_line_width_symbol();
 	do {
 		double xcm, ycm;
 		for (unsigned int i = 0; i < _num_xmatrix_data; i++) {
@@ -1322,8 +1378,13 @@ bool
 draw_isopycnalCmd()
 {
 	if (!scales_defined()) {
+#if 0				// Fix SF bug #129856 (I hope!)
 		no_scales_error();
 		return false;
+#else
+		create_x_scale();
+		create_y_scale();
+#endif
 	}
 	bool labelled = true;
 	double density, P_sigma = 0.0, P_theta = 0.0;
@@ -1390,10 +1451,27 @@ draw_isopycnalCmd()
 	bool first = true;
 	double Tlast = -999;
 	double S, T = -999.0;
-	double dS = (Smax - Smin) / 100;
+
+	// Next few lines determine how fine a mesh to use for S,
+	// since a fine mesh is needed if the lines are nearly
+	// vertical, but a coarser mesh is more efficient.
+	double d_rho_dS = (rho(_xleft, _ybottom, 0.0) - rho(_xright, _ybottom, 0.0)) / (_xright - _xleft);
+
+	double d_rho_dT = (rho(_xleft, _ytop, 0.0) - rho(_xleft, _ybottom, 0.0)) / (_ytop - _ybottom);
+
+	double slope_on_page = d_rho_dS*(_xright-_xleft)/(d_rho_dT*(_ytop-_ybottom));
+	double dS;
+	// BUG: the S increments are just a guess that seems ok now.
+	if (slope_on_page > 5.0) 
+		dS = (Smax - Smin) / 500.0;
+	else 
+		dS = (Smax - Smin) / 100.0;
+
+	//printf("rho_x %f   rho_y %f   slope %f\n", d_rho_dS, d_rho_dT, d_rho_dS*(_xright-_xleft)/(d_rho_dT*(_ytop-_ybottom)));
+
 	double S_cm, T_cm, S_cm_last = 0, T_cm_last = 0;
-	vector<double> S_label_cm;	// where to put labels
-	vector<double> T_label_cm;
+	std::vector<double> S_label_cm;	// where to put labels
+	std::vector<double> T_label_cm;
 	double cum_dist = 0.0;
 	extern double _contour_space_first, _contour_space_later;
 	double contour_space_first, contour_space_later;
@@ -1552,8 +1630,13 @@ draw_image_histogramCmd()
 	double          left, right;
 	int             num;
 	if (!scales_defined()) {
+#if 0				// Fix SF bug #129856 (I hope!)
 		no_scales_error();
 		return false;
+#else
+		create_x_scale();
+		create_y_scale();
+#endif
 	}
 	if (!image_exists()) {
 		err("First `read image' or `convert grid to image'");
@@ -1641,7 +1724,7 @@ draw_image_paletteCmd()
 	int             otherside=0;
 	int             i;
 	double          left_cm, bottom_cm, right_cm, top_cm;
-	int             words_understood = 3;	// 3 in `draw image palette'
+	unsigned int    words_understood = 3;	// 3 in `draw image palette'
 	unsigned char   gray[LEN];
 	double          left, inc = 0.0, right, grayVal, grayInc;
 	int             num;
@@ -1651,8 +1734,13 @@ draw_image_paletteCmd()
 	if (!get_var("..xsize..", &dx))
 		warning("Sorry, don't know value of ..xsize.. so using XSIZE_DEFAULT");
 	if (!scales_defined()) {
+#if 0				// Fix SF bug #129856 (I hope!)
 		no_scales_error();
 		return false;
+#else
+		create_x_scale();
+		create_y_scale();
+#endif
 	}
 	// Check for old usage `draw image grayscale'.  Give warning, but proceed
 	// anyway.
@@ -1793,30 +1881,37 @@ draw_image_paletteCmd()
 			     left_cm, bottom_cm, right_cm, top_cm,
 			     false);
 	}
-	fprintf(_grPS, "S Q %% turn clipping off for image palette\n");
+	fprintf(_grPS, "Q %% turn clipping off for image palette\n");
+	double actual_linewidth = _griState.linewidth_line();
+	_griState.set_linewidth_line(_griState.linewidth_axis());
 	if (rotpal==0) {
 		if (otherside==0)
 			gr_drawxaxis(0.0, left, inc, right, gr_axis_BOTTOM);
 		else
 			gr_drawxaxis(1.0, left, inc, right, gr_axis_TOP);
+		fprintf(_grPS, "%.3f w %% test\n", _griState.linewidth_axis());
 		GriPath p(4);
 		p.push_back(left, 0.0, 'm');
 		p.push_back(left, 1.0, 'l');
 		p.push_back(right, 1.0, 'l');
 		p.push_back(right, 0.0, 'l');
+		p.push_back(left, 0.0, 'l');
 		p.stroke(units_user);
 	} else {
 		if (otherside==0)
 			gr_drawyaxis(0.0, left, inc, right, gr_axis_LEFT);
 		else
 			gr_drawyaxis(1.0, left, inc, right, gr_axis_RIGHT);
+		fprintf(_grPS, "%.3f w\n", _griState.linewidth_axis());
 		GriPath p(4);
 		p.push_back(0.0, left, 'm');
 		p.push_back(1.0, left, 'l');
 		p.push_back(1.0, right, 'l');
 		p.push_back(0.0, right, 'l');
+		p.push_back(0.0, left, 'l');
 		p.stroke(units_user);
 	}
+	_griState.set_linewidth_line(actual_linewidth);
 	return true;
 }
 
@@ -1831,9 +1926,13 @@ draw_imageCmd()
 		return false;
 	}
 	if (!scales_defined()) {
-		demonstrate_command_usage();
+#if 0				// Fix SF bug #129856 (I hope!)
 		no_scales_error();
 		return false;
+#else
+		create_x_scale();
+		create_y_scale();
+#endif
 	}
 	if (!image_exists()) {
 		demonstrate_command_usage();
@@ -1881,24 +1980,28 @@ draw_labelCmd()
 	double          textangle_deg = 0.0;
 	double          tmp1, tmp2;
 	double          xcm, ycm;
-	int             QuoteEnd;
 	bool            user_units;
 	bool            left_justified = false, centered = false, right_justified = false;
 	int             coord_word = 0;
-	GriString       label(_cmdLine);
 	if (_nword < 5) {
 		NUMBER_WORDS_ERROR;
 		demonstrate_command_usage();
 		return false;
 	}
 	// Find quote string
-	if (!(QuoteEnd = ExtractQuote(label.getValue(), _cmdLine))) {
-		demonstrate_command_usage();
-		err("Missing \"string\".");
+	std::string unquoted;
+	int status = ExtractQuote(_cmdLine, unquoted);
+	if (status == 0) {
+		err("`draw label' needs a double-quoted string");
 		return false;
 	}
-	strcpy(_cmdLineCOPY, _cmdLine);
-	chop_into_words(_cmdLineCOPY + QuoteEnd, _word, &_nword, MAX_nword);
+	if (status < 0) {
+		err("`draw label' found start of a double-quoted string, but not a closing double-quote");
+		return false;
+	}
+	int QuoteEnd = status;
+	GriString label(unquoted.c_str());
+	chop_into_words(_cmdLine + QuoteEnd, _word, &_nword, MAX_nword);
 	// Parse for text angle, in degrees from horizontal.  If the option
 	// [rotated .deg.] exists, interpret and then strip from the commandline.
 	if (1 == get_cmd_values(_word, _nword, "rotated", 1, _dstack)) {
@@ -1925,16 +2028,20 @@ draw_labelCmd()
 		coord_word += 2;
 	} else {
 		demonstrate_command_usage();
-		err("Where do you want this label drawn?");
+		err("Where do you want this label drawn?  (missing `at' word)");
 		return false;
 	}
 	user_units = strcmp(_word[_nword - 1], "cm") ? true : false;
 	set_environment();
 	if (user_units) {
 		if (!scales_defined()) {
-			demonstrate_command_usage();
+#if 0				// Fix SF bug #129856 (I hope!)
 			no_scales_error();
 			return false;
+#else
+			create_x_scale();
+			create_y_scale();
+#endif
 		}
 		if (!getdnum(_word[coord_word], &tmp1)) {
 			demonstrate_command_usage();
@@ -1974,7 +2081,7 @@ draw_labelCmd()
 	} else if (centered) {
 		label.draw(xcm, ycm, TEXT_CENTERED, textangle_deg);
 	} else {
-		err("Where do you want this label drawn?");
+		err("Where do you want this label drawn?"); // never reached, I think
 		return false;
 	}
 	return true;
@@ -2018,8 +2125,13 @@ draw_line_from_toCmd()
 	} else {
 		// User units -- draw axes if needed
 		if (!scales_defined()) {
+#if 0				// Fix SF bug #129856 (I hope!)
 			no_scales_error();
 			return false;
+#else
+			create_x_scale();
+			create_y_scale();
+#endif
 		}
 		if (!gr_missingx(x0) && !gr_missingx(x1) && !gr_missingy(y0) && !gr_missingy(y1)) {
 			p.push_back(x0, y0, 'm');
@@ -2101,8 +2213,13 @@ draw_patchesCmd()
 		return true;
 	}
 	if (!scales_defined()) {
+#if 0				// Fix SF bug #129856 (I hope!)
 		no_scales_error();
 		return false;
+#else
+		create_x_scale();
+		create_y_scale();
+#endif
 	}
 	if (!_imageTransform_exists) {
 		err("Image transform doesn't exist yet.  First `set image grayscale'");
@@ -2233,19 +2350,26 @@ draw_titleCmd()
 {
 	if (_nword > 2) {
 		// find quote string 
-		if (!ExtractQuote(_grTempString, _cmdLine)) {
-			err("`draw title' needs \"string\"");
+		std::string unquoted;
+		int status = ExtractQuote(_cmdLine, unquoted);
+		if (status == 0) {
+			err("`draw title' found no double-quoted string to use.");
 			return false;
 		}
-		if (strlen(_grTempString) > 0) {
-			double          xmargin = XMARGIN_DEFAULT, xsize = XSIZE_DEFAULT;
+		if (status < 0) {
+			err("`draw title' found no starting double-quote, but no ending double-quote.");
+			return false;
+		}
+		if (!unquoted.empty()) {
+			double xmargin = XMARGIN_DEFAULT;
+			double xsize   = XSIZE_DEFAULT;
 			set_environment();
 			if (!get_var("..xmargin..", &xmargin))
 				warning("Sorry, don't know value of ..xmargin.. so using XMARGIN_DEFAULT");
 			if (!get_var("..xsize..", &xsize))
 				warning("Sorry, don't know value of ..xsize.. so using XSIZE_DEFAULT");
 			GriString label;
-			label.fromSTR(_grTempString);
+			label.fromSTR(unquoted.c_str());
 			label.draw(xmargin + 0.5 * xsize,
 				   _top_of_plot + OFFSET_AFTER_TITLE,
 				   TEXT_CENTERED,
@@ -2268,11 +2392,16 @@ draw_valuesCmd()
 		return false;
 	}
 	if (!scales_defined()) {
+#if 0				// Fix SF bug #129856 (I hope!)
 		no_scales_error();
 		return false;
+#else
+		create_x_scale();
+		create_y_scale();
+#endif
 	}
 	// Get separation x and y if they are given
-	int gave_separation = 0;
+	unsigned int gave_separation = 0;
 	double xseparation = 0.0, yseparation = 0.0;
 	if (2 == get_cmd_values(_word, _nword, "separation", 2, _dstack)) {
 		xseparation = _dstack[0];
@@ -2285,10 +2414,10 @@ draw_valuesCmd()
 		gave_separation = 1;
 	}
 	// Get format if it is given
-	int             gave_fmt = 0;
+	unsigned int    gave_fmt = 0;
 	char            fmt[20];
 	strcpy(fmt, "%g");
-	for (int word = 2; word < _nword; word++) {
+	for (unsigned int word = 2; word < _nword; word++) {
 		if ('%' == *_word[word]) {
 			strcpy(fmt, _word[word]);
 			gave_fmt = 1;
@@ -2473,8 +2602,13 @@ draw_x_box_plotCmd()
 		return true;
 	}
 	if (!scales_defined()) {
+#if 0				// Fix SF bug #129856 (I hope!)
 		no_scales_error();
 		return false;
+#else
+		create_x_scale();
+		create_y_scale();
+#endif
 	}
 	// Check command syntax
 	switch (_nword) {
@@ -2516,6 +2650,8 @@ draw_x_box_plotCmd()
 	ymax_cm = y_cm - size_cm / 2.0;
 	static GriPath p(7);
 	p.clear();
+#if 0
+	// Clockwise from upper-left
 	p.push_back(q1_cm, ymax_cm, 'm');
 	p.push_back(q3_cm, ymax_cm, 'l');
 	p.push_back(q3_cm, ymin_cm, 'l');
@@ -2525,6 +2661,19 @@ draw_x_box_plotCmd()
 	p.push_back(q2_cm, ymin_cm, 'l');
 	set_line_width_symbol();
 	p.stroke(units_cm);
+#else
+	// Clockwise from upper-left
+	set_line_width_curve();
+	p.push_back(q1_cm, ymax_cm, 'm');
+	p.push_back(q3_cm, ymax_cm, 'l');
+	p.push_back(q3_cm, ymin_cm, 'l');
+	p.push_back(q1_cm, ymin_cm, 'l');
+	p.stroke(units_cm, -1.0, true);
+	p.clear();
+	p.push_back(q2_cm, ymax_cm, 'm');
+	p.push_back(q2_cm, ymin_cm, 'l');
+	p.stroke(units_cm);
+#endif
 	// draw whiskers
 	upper_adjacent_value = q3 - 1;	// will be max between q3 and upper 
 	lower_adjacent_value = q1 + 1;	// will be min between q3 and upper 
@@ -2608,8 +2757,13 @@ draw_y_box_plotCmd()
 		return true;
 	}
 	if (!scales_defined()) {
+#if 0				// Fix SF bug #129856 (I hope!)
 		no_scales_error();
 		return false;
+#else
+		create_x_scale();
+		create_y_scale();
+#endif
 	}
 	// Check command syntax
 	switch (_nword) {
@@ -2650,7 +2804,9 @@ draw_y_box_plotCmd()
 	xmin_cm = x_cm + size_cm / 2.0;
 	xmax_cm = x_cm - size_cm / 2.0;
 	static GriPath p(7);
+#if 0
 	p.clear();
+	set_line_width_curve();
 	p.push_back(xmin_cm, q1_cm, 'm');
 	p.push_back(xmax_cm, q1_cm, 'l');
 	p.push_back(xmax_cm, q3_cm, 'l');
@@ -2658,8 +2814,20 @@ draw_y_box_plotCmd()
 	p.push_back(xmin_cm, q1_cm, 'l');
 	p.push_back(xmin_cm, q2_cm, 'm');
 	p.push_back(xmax_cm, q2_cm, 'l');
-	set_line_width_symbol();
 	p.stroke(units_cm);
+#else
+	p.clear();
+	set_line_width_curve();
+	p.push_back(xmin_cm, q1_cm, 'm');
+	p.push_back(xmax_cm, q1_cm, 'l');
+	p.push_back(xmax_cm, q3_cm, 'l');
+	p.push_back(xmin_cm, q3_cm, 'l');
+	p.stroke(units_cm, -1.0, true);
+	p.clear();
+	p.push_back(xmin_cm, q2_cm, 'm');
+	p.push_back(xmax_cm, q2_cm, 'l');
+	p.stroke(units_cm);
+#endif
 	// Draw whiskers.
 	upper_adjacent_value = q3 - 1; // will be max between q3 and upper 
 	lower_adjacent_value = q1 + 1; // will be min between q3 and upper 
@@ -2729,8 +2897,13 @@ bool
 draw_zero_lineCmd()
 {
 	if (!scales_defined()) {
+#if 0				// Fix SF bug #129856 (I hope!)
 		no_scales_error();
 		return false;
+#else
+		create_x_scale();
+		create_y_scale();
+#endif
 	}
 	switch (_nword) {
 	case 3:
@@ -2823,15 +2996,7 @@ set_y_scale()
 bool
 draw_gri_logoCmd()
 {
-#if 1
-	double dx, dy;
-	warning("DEBUG: using .dx. and .dy. temporarily.  draw_gri_logoCmd()/draw.cc");
-	if (!get_var(".dx.", &dx))
-		err("DEBUG: cannot find .dx.");
-	if (!get_var(".dy.", &dy))
-		err("DEBUG: cannot find .dy.");
-#endif
-	printf("DEBUG: dx=%f dy=%f cm LATER HARD-WIRE IN.\n", dx, dy);
+	double dx=0.03, dy=0.03; // Q: should these be arguments?
 	//
 	// Process args
 	double x_cm, y_cm, height_cm;
