@@ -1,12 +1,20 @@
 #include <string>
+#include <string.h>
+
+#if !defined(IS_MINGW32)
+#include <strings.h>
+#else
+#define index strrchr
+#endif
+
 #include <math.h>
 #include <stdlib.h>
 #include "gr.hh"
 #include "GriPath.hh"
 #include "extern.hh"
 
-#if defined(__DECCXX)
-extern "C" char *index(const char *s, int c); // DEC-cxx needs this (dunno why)
+#if defined(__DECCXX) || defined(OS_IS_BEOS)
+extern "C" char *index(const char *s, int c);
 #endif
 
 #define TIC_RATIO	0.5	/* (Length small tic) / (large tic) */
@@ -115,6 +123,7 @@ void
 gr_drawxaxis(double y, double xl, double xinc, double xr, gr_axis_properties side)
 {
 	GriString label;
+	std::string slabel;
 	extern char     _xtype_map;
 	double          CapHeight = gr_currentCapHeight_cm();
 	double          angle = 0.0;	// angle of axis tics, labels, etc
@@ -199,28 +208,35 @@ gr_drawxaxis(double y, double xl, double xinc, double xr, gr_axis_properties sid
 						// over-ride any existing format
 						int             hour, min, sec;
 						if (gr_multiple(next, 1.0, 1.0e-6)) {
-							hour = (int)floor(1.0e-4 + next);
-							sprintf(_grTempString, "%d$\\circ$%c", hour, _xtype_map);
+							hour = (int)floor(1.0e-4 + fabs(next));
+							if (next >= 0.0) 
+								sprintf(_grTempString,"%d$\\circ$%c",hour,_xtype_map);
+							else
+								sprintf(_grTempString,"-%d$\\circ$%c",hour,_xtype_map);
 						} else if (gr_multiple(next, 1.0 / 60.0, 1.0e-7)) {
-							hour = (int)floor(1.0e-4 + next);
-							min = (int)floor(1e-5 + 60.0 * (next - hour));
-							sprintf(_grTempString, "%d$\\circ$%d'%c", hour, min, _xtype_map);
+							hour = (int)floor(1.0e-4 + fabs(next));
+							min = (int)floor(1e-5 + 60.0 * (fabs(next) - hour));
+							if (next >= 0.0)
+								sprintf(_grTempString,"%d$\\circ$%d'%c",hour,min,_xtype_map);
+							else 
+								sprintf(_grTempString,"-%d$\\circ$%d'%c",hour,min,_xtype_map);
 						} else if (gr_multiple(next, 1.0 / 3600.0, 1.0e-8)) {
-							hour = (int)floor(1.0e-4 + next);
-							min = (int)floor(1e-5 + 60.0 * (next - hour));
-							sec = (int)floor(1e-5 + 3600.0 * (next - hour - min / 60.0));
-							sprintf(_grTempString, "%d$\\circ$%d'%d\"%c", hour, min, sec, _xtype_map);
-						} else
-							sprintf(_grTempString, "%f$\\circ$%c", next, _xtype_map);
+							hour = (int)floor(1.0e-4 + fabs(next));
+							min = (int)floor(1e-5 + 60.0 * (fabs(next) - hour));
+							sec = (int)floor(1e-5 + 3600.0 * (fabs(next) - hour - min / 60.0));
+							if (next >= 0.0)
+								sprintf(_grTempString, "%d$\\circ$%d'%d\"%c",hour,min,sec,_xtype_map);
+							else 
+								sprintf(_grTempString, "-%d$\\circ$%d'%d\"%c",hour,min,sec,_xtype_map);
+						} else {
+							sprintf(_grTempString,"%f$\\circ$%c",next,_xtype_map);
+						}
 					} else if (strlen(_grNumFormat_x)) {
-						if (fabs(next / xinc) > 1.0e-5)
-							sprintf(_grTempString, _grNumFormat_x, next);
-						else
-							sprintf(_grTempString, "0");
+						sprintf(_grTempString, _grNumFormat_x, next);
 						if (get_flag("emulate_gre")) {
 							char *e = index(_grTempString, int('E'));
 							if (e != NULL) {
-								string gs(_grTempString);
+								std::string gs(_grTempString);
 								size_t chop;
 								if (STRING_NPOS != (chop = gs.find("E+0"))) {
 									gs.replace(chop, 3, "$\\times10^{");
@@ -241,11 +257,14 @@ gr_drawxaxis(double y, double xl, double xinc, double xr, gr_axis_properties sid
 								strcpy(_grTempString, gs.c_str());
 							}
 						}
-					} else
+					} else {
 						*_grTempString = '\0';
+					}
 					// Text is rotated
 					angle -= 90.0 / DEG_PER_RAD;
-					label.fromSTR(_grTempString);
+					slabel.assign(_grTempString);
+					fix_negative_zero(slabel);
+					label.fromSTR(slabel.c_str());
 					label.draw(xcm - offset * sin(angle),
 						   ycm + offset * cos(angle),
 						   TEXT_CENTERED,
@@ -290,7 +309,9 @@ gr_drawxaxis(double y, double xl, double xinc, double xr, gr_axis_properties sid
 						sprintf(_grTempString, "1");
 					else
 						sprintf(_grTempString, "$10^{%.0f}$", log10(next));
-					label.fromSTR(_grTempString);
+					slabel.assign(_grTempString);
+					fix_negative_zero(slabel);
+					label.fromSTR(slabel.c_str());
 					label.draw(xcm, ycm + offset, TEXT_CENTERED, 0.0);
 				}
 			} else {
@@ -360,6 +381,7 @@ void
 gr_drawyaxis(double x, double yb, double yinc, double yt, gr_axis_properties side)
 {
 	GriString label;
+	std::string slabel;
 	extern char     _ytype_map;
 	double          CapHeight = gr_currentCapHeight_cm();
 	double          angle = 0.0; // angle of axis tics, labels, etc
@@ -427,33 +449,35 @@ gr_drawyaxis(double x, double yb, double yinc, double yt, gr_axis_properties sid
 						// over-ride any existing format
 						int             hour, min, sec;
 						if (gr_multiple(next, 1.0, 1.0e-6)) {
-							hour = (int)floor(1.0e-4 + next);
-							sprintf(_grTempString, "%d$\\circ$%c", hour, _ytype_map);
+							hour = (int)floor(1.0e-4 + fabs(next));
+							if (next >= 0.0)
+								sprintf(_grTempString,"%d$\\circ$%c",hour,_ytype_map);
+							else
+								sprintf(_grTempString,"-%d$\\circ$%c",hour,_ytype_map);
 						} else if (gr_multiple(next, 1.0 / 60.0, 1.0e-7)) {
-							hour = (int)floor(1.0e-4 + next);
-							min = (int)floor(1e-5 + 60.0 * (next - hour));
-							sprintf(_grTempString, "%d$\\circ$%d'%c", hour, min, _ytype_map);
+							hour = (int)floor(1.0e-4 + fabs(next));
+							min = (int)floor(1e-5 + 60.0 * (fabs(next) - hour));
+							if (next >= 0.0) 
+								sprintf(_grTempString,"%d$\\circ$%d'%c",hour,min,_ytype_map);
+							else 
+								sprintf(_grTempString,"-%d$\\circ$%d'%c",hour,min,_ytype_map);
 						} else if (gr_multiple(next, 1.0 / 3600.0, 1.0e-8)) {
-							hour = (int)floor(1.0e-4 + next);
-							min = (int)floor(1e-5 + 60.0 * (next - hour));
-							sec = (int)floor(1e-5 + 3600.0 * (next - hour - min / 60.0));
-							sprintf(_grTempString, "%d$\\circ$%d'%d\"%c", hour, min, sec, _ytype_map);
+							hour = (int)floor(1.0e-4 + fabs(next));
+							min = (int)floor(1e-5 + 60.0 * (fabs(next) - hour));
+							sec = (int)floor(1e-5 + 3600.0 * (fabs(next) - hour - min / 60.0));
+							if (next >= 0.0)
+								sprintf(_grTempString, "%d$\\circ$%d'%d\"%c",hour,min,sec,_ytype_map);
+							else 
+								sprintf(_grTempString, "-%d$\\circ$%d'%d\"%c",hour,min,sec,_ytype_map);
 						} else {
-							sprintf(_grTempString, "%f$\\circ$%c", next, _ytype_map);
+							sprintf(_grTempString,"%f$\\circ$%c",next,_ytype_map);
 						}
 					} else if (strlen(_grNumFormat_y)) {
 						if (get_flag("emulate_gre")) {
-#if 0
-							if (fabs(next / yinc) > 1.0e-5)
-								sprintf(_grTempString, _grNumFormat_y, next);
-							else
-								sprintf(_grTempString, "0         ");
-#else
 							sprintf(_grTempString, _grNumFormat_y, next);
-#endif
 							char *e = index(_grTempString, int('E'));
 							if (e != NULL) {
-								string gs(_grTempString);
+								std::string gs(_grTempString);
 								size_t chop;
 								if (STRING_NPOS != (chop = gs.find("E+0"))) {
 									gs.replace(chop, 3, "$\\times10^{");
@@ -474,19 +498,14 @@ gr_drawyaxis(double x, double yb, double yinc, double yt, gr_axis_properties sid
 								strcpy(_grTempString, gs.c_str());
 							}
 						} else {
-#if 0
-							if (fabs(next / yinc) > 1.0e-5)
-								sprintf(_grTempString, _grNumFormat_y, next);
-							else
-								sprintf(_grTempString, "0");
-#else
 							sprintf(_grTempString, _grNumFormat_y, next);
-#endif
 						}
 					} else {
 						*_grTempString = '\0';
 					}
-					label.fromSTR(_grTempString);
+					slabel.assign(_grTempString);
+					fix_negative_zero(slabel);
+					label.fromSTR(slabel.c_str());
 					if (side == gr_axis_LEFT) {
 						label.draw(labelx_cm, labely_cm, TEXT_RJUST, angle * DEG_PER_RAD);
 					} else {
@@ -537,7 +556,9 @@ gr_drawyaxis(double x, double yb, double yinc, double yt, gr_axis_properties sid
 						sprintf(_grTempString, "1");
 					else
 						sprintf(_grTempString, "$10^{%.0f}$", log10(next));
-					label.fromSTR(_grTempString);
+					slabel.assign(_grTempString);
+					fix_negative_zero(slabel);
+					label.fromSTR(slabel.c_str());
 					if (side == gr_axis_LEFT)
 						label.draw(xcm2 + offset, ycm2 - 0.5 * CapHeight, TEXT_RJUST, 0.0);
 					else
