@@ -14,8 +14,32 @@
 #include        "GriColor.hh"
 #include        "superus.hh"
 
-static vector <GriNamedColor> colorStack;
+#if defined(HAVE_GETHOSTNAME)
+#if defined(IS_SUN)
+extern "C" unsigned int gethostname(char *name, int namelen);
+#endif
+#endif
 
+#if defined(OS_IS_BEOS)
+#include <netdb.h>
+#endif
+static std::vector<GriNamedColor> colorStack;
+
+typedef struct {
+	unsigned int code;
+	char *action;
+} superuser_flag;
+static superuser_flag sflag[] = {
+	{FLAG_SYN,  "Print cmdline before/after sub synonyms"},
+	{FLAG_RPN,  "Print cmdline before/after sub rpn"},
+	{FLAG_NEW,  "Print new commands being defined"},
+	{FLAG_SYS,  "Print system commands before passing"},
+	{FLAG_AUT2, "Variable; for use by developers only"},
+	{FLAG_AUT1, "Variable; for use by developers only"},
+	{0, NULL}
+};
+static std::string psname("");
+static bool user_gave_directory = false;
 
 extern char     _grTempString[];
 
@@ -32,7 +56,7 @@ static void     create_builtin_synonyms(void);
 static void     set_defaults(void);
 static int      interpret_optional_arguments(int argc, char *argv[]);
 static void     get_input_simulation(int argc, char *argv[], int separator);
-static void     insert_creator_name_in_PS(int argc, char *argv[], const string&psname);
+static void     insert_creator_name_in_PS(int argc, char *argv[], const std::string&psname);
 static void     dogrirc(void);
 #if 0
 static void     show_startup_msg(void);
@@ -64,6 +88,7 @@ create_color(const char *name, double r, double g, double b)
 	return true;
 }
 
+#if 0
 // RETURN value contents of s following the last ',' in s, or all of s
 static          char*
 last_name(char *s)
@@ -90,10 +115,17 @@ last_name(char *s)
 	}
 	return return_value;
 }
+#endif
 
 bool
 start_up(int argc, char **argv)
 {
+#if 0
+	{
+		printf("ARGS:\n");
+		for (int i = 0; i < argc; i++) printf("\t'%s'\n", argv[i]);
+	}
+#endif
 	// Record version number
 	int major_version, minor_version, minor_minor_version;
 	extern char _input_data_separator; // defined in gri.cc
@@ -123,54 +155,93 @@ start_up(int argc, char **argv)
 	PUT_VAR("..eof..", 0.0);
 	PUT_VAR("..landscape..", 0.0);
 	PUT_VAR("..publication..", 0.0);
-	PUT_VAR("..lastx..", 0.0);
-	PUT_VAR("..lasty..", 0.0);
+	PUT_VAR("..xlast..", 0.0);
+	PUT_VAR("..ylast..", 0.0);
 	PUT_VAR("..image_width..", 0.0);
 	PUT_VAR("..image_height..", 0.0);
 	last_optional_arg = interpret_optional_arguments(argc, argv);
+
+	//printf("DEBUG1 last_optional_arg= %d\n", last_optional_arg);
+
 	// Get name of PostScript file if it was provided.  Must do this before
 	// getting cmd file because of demands of gr_setup.  Also, check here for
 	// input_simulation words.
-	int separator = argc;
-	for (int i = argc - 1; i > -1; i--)
-		if (!strcmp(argv[i], SEPARATOR))
-			separator = i;
+	int separator = last_optional_arg;
+	if (last_optional_arg == argc - 1) {
+		_margin.assign("  ");
+		push_cmd_file("stdin", batch() ? false : true, true, "r");
+	} else {
+		separator++;
+		std::string fname(argv[1 + last_optional_arg]);
+
+		// If filename shorter than 4 characters, cannot have .gri suffix,
+		// so append it.
+		std::string::size_type p = fname.rfind(".gri");
+		if (fname.size() < 4 || p != -4 + fname.size())
+			fname.append(".gri");
+
+		// If user didn't give psname, create it.  First, must trim the
+		// ".gri" suffix (which is sure to be there).  Then, must remove
+		// any filename path, since we want the .ps file to be created
+		// in this local directory .
+		if (psname.empty()) {
+			psname = fname;
+			int l = psname.size();
+			psname.STRINGERASE(l - 4, l - 1);
+			std::string::size_type last_slash = psname.rfind("/");
+			if (last_slash != STRING_NPOS)
+				psname.STRINGERASE(0, last_slash + 1);
+			psname.append(".ps");
+		}
+		if (!push_cmd_file(fname.c_str(), false, false, "r")) {
+			fprintf(stderr, "Cannot open commandfile `%s'\n", fname.c_str());
+			delete_ps_file();
+			gri_exit(1);
+		}
+
+		// Possibly they gave a ps filename ...
+		if (argc > last_optional_arg + 2) {
+			std::string tmp(argv[last_optional_arg+2]);
+			std::string::size_type p = tmp.rfind(".ps");
+			if (p != STRING_NPOS && p == -3 + tmp.size()) {
+				warning("\
+first argument looks like a PostScript filename.  Older versions\n\
+         of Gri allowed you to specify the PostScript name that way,\n\
+         but now you must use the \"-output\" option, as for example:\n\
+             gri ... -output \\", tmp.c_str(), " ...\n\
+         As it is, Gri is using the filename `", psname.c_str(), "'.", "\\");
+				//psname = tmp;
+			}
+		}
+		gr_setup_ps_filename(psname.c_str());
+	}
+
+	//printf("DEBUG2 separator= %d\n", separator);
 	get_input_simulation(argc, argv, separator);
 
-	string psname("");
-	if (separator == 3 + last_optional_arg) {
-		// Cmdline has both cmdfile name and psfile name
-		if (!strcmp(argv[last_optional_arg + 1], argv[last_optional_arg + 2]))
-			fatal_err("Gri requires Command filename to differ from PostScript filename");
-		psname = last_name(argv[last_optional_arg + 2]);
-		gr_setup_ps_filename(psname.c_str());
-	} else if (separator == 2 + last_optional_arg) {
-		// Only have cmdname, so have to make up a psname
-		psname = last_name(argv[last_optional_arg + 1]);
-		//printf("psname -> '%s'\n", psname.c_str());
-		string::size_type slash_location = psname.find_last_of("/");
-		if (slash_location != STRING_NPOS)
-			psname.STRINGERASE(0, slash_location+1);
-		//printf("psname -> '%s'\n", psname.c_str());
-		unsigned int l = psname.length();
-		if (l > 4
-		    && psname[l-1] == 'i'
-		    && psname[l-2] == 'r' 
-		    && psname[l-3] == 'g'
-		    && psname[l-4] == '.') {
-			psname.STRINGERASE(l-4,4);
-		}
-		//printf("psname -> '%s'\n", psname.c_str());
-		psname.append(".ps");
-		//printf("psname -> '%s'\n", psname.c_str());
-		gr_setup_ps_filename(psname.c_str());
-	} else {
-		// Let Gri make a default name, of the form gr.ps##
-		psname = "";
-	}
 	insert_creator_name_in_PS(argc, argv, psname);
 	// Finally, ready to begin plot.
 	gr_begin(1);
+
+	// Embed info on how gri was invoked.
+	extern FILE *_grPS;
+	char host[BUFSIZ];
+#if defined(HAVE_GETHOSTNAME)
+	if (0 != gethostname(host, BUFSIZ - 1))
+		strcpy(host, "unknown");
+#else
+	strcpy(host, "unknown");
+#endif
+	fprintf(_grPS, "%%gri:# Gri was invoked by user named\n%%gri:#     %s\n%%gri:# on host named\n%%gri:#     %s\n%%gri:# using the command\n%%gri:#   ", egetenv("USER"), host);
+	for (int i = 0; i < argc; i++)
+		fprintf(_grPS, " %s", argv[i]);
+	SECOND_TYPE sec;
+	time(&sec);
+	sprintf(_grTempString, "%s", asctime(localtime(&sec)));
+	_grTempString[-1 + strlen(_grTempString)] = '\0'; // trim newline
+	fprintf(_grPS, "\n%%gri:# at time %s.\n", _grTempString);
+
+	put_syn("\\.ps_file.", gr_currentPSfilename(), true);
 	// Disable tracing during startup phase, unless in superuser mode.
 	double          trace_old;
 	get_var("..trace..", &trace_old);
@@ -179,119 +250,76 @@ start_up(int argc, char **argv)
 	// the searching for the gri.cmd file.
 	bool tmp = _store_cmds_in_ps;
 	_store_cmds_in_ps = false;
-	if (((unsigned) superuser()) & FLAG_AUT2)
+	if (superuser() & FLAG_AUT2)
 		printf("Processing gri.cmd ...");
-	create_commands(GRI_COMMANDS_FILE);
-	if (((unsigned) superuser()) & FLAG_AUT2)
+	create_commands(GRI_COMMANDS_FILE, user_gave_directory);
+	if (superuser() & FLAG_AUT2)
 		printf(" done\n");
 	create_builtin_colors();
+
+	// Define default paths
+	char* griinputs = egetenv("GRIINPUTS");
+	if (griinputs != NULL && strlen(griinputs) > 0) {
+		warning("Your GRIINPUTS environment variable has been IGNORED.\n         In this version of gri, you must put the line\n             set path \"\\",
+			griinputs, "\" for commands\n         in your ~/.grirc file to get the same effect.\n", "\\");
+	}
+
+	put_syn("\\.path_data.", ".", true);
+	put_syn("\\.path_commands.", ".", true);
+
 	// Do user's ~/.grirc file.
 	_store_cmds_in_ps = tmp;
 	dogrirc();
-#if 0				// 1999-11-27 do instead in doline.cc
-	// Show startup msg
-	if (_no_startup_message == false && !batch())
-		show_startup_msg();
-#endif
+
 	// Re-enable tracing
 	PUT_VAR("..trace..", trace_old);
 
 	// DataFile stack
 	DataFile new_data_file;
+	if (superuser() & FLAG_AUT1)printf("\nDEBUG: %s:%d pushing back a datafile at address %x\n",__FILE__,__LINE__,int(&new_data_file));
 	_dataFILE.push_back(new_data_file);
 
-    // Figure out if there was a cmdfile supplied on options line.
-	if (argc < 2 + last_optional_arg) {
-		// There was no cmdfile given, so take commands from stdin
-		_margin.assign("  ");
-		push_cmd_file("stdin", batch() ? false : true, true, "r");
-	} else if (argc == 2 + last_optional_arg
-		   || argc == 3 + last_optional_arg) {
-		// Command-file name given, but no postscript-file given
-		string fname(argv[1 + last_optional_arg]);
-		// Ensure that it's not a 'gre' commandfile
-		string::size_type p = fname.rfind(".gre");
-		if (p == -4 + fname.size()) {
-			fprintf(stderr, "Error: gri cannot execute gre files (i.e. files ending in .gre)\n");
-			delete_ps_file();
-			gri_exit(1);
-		} else {
-			// If filename shorter than 4 characters, cannot have .gri suffix,
-			// so append it.
-			if (fname.size() < 4) {
-				fname.append(".gri");
-			} else {
-				string::size_type p = fname.rfind(".gri");
-				if (p != -4 + fname.size())
-					fname.append(".gri");
-			}
-			if (!push_cmd_file(fname.c_str(), false, false, "r")) {
-				fprintf(stderr, "Cannot open commandfile `%s'\n", fname.c_str());
-				delete_ps_file();
-				gri_exit(1);
-			}
-		}
-	} else {
-		delete_ps_file();
-		fatal_err("Too many items on command line (2 allowed): `\\",
-			  argv[1 + last_optional_arg], "', `",
-			  argv[2 + last_optional_arg], "', `",
-			  argv[3 + last_optional_arg], "' ...", "\\");
-		return false;           // never executed
-	}
 	_first = true;
-	_bounding_box.set(0., 0., 0., 0.);
+	_bounding_box.set(0.0, 0.0, 0.0, 0.0);
+	extern rectangle _page_size;
+	_page_size.set(0.0, 0.0, 0.0, 0.0);
+	//printf("At end of start_up, _cmdLine is <%s>\n",_cmdLine);
 	return true;
 }
 
 static void
 create_builtin_colors()
 {
-	GriNamedColor       color;
-
-	color.setNameRGB(		"white",		1.000,	1.000,	1.000);
+	GriNamedColor color;
+	color.setNameRGB("white",		1.000,	1.000,	1.000);
 	colorStack.push_back(color);
-
-	color.setNameRGB(		"LightGray",		0.827,	0.827,	0.827);
+	color.setNameRGB("LightGray",		0.827,	0.827,	0.827);
 	colorStack.push_back(color);
-
-	color.setNameRGB(		"darkslategray",	0.184,	0.310,	0.310);
+	color.setNameRGB("darkslategray",	0.184,	0.310,	0.310);
 	colorStack.push_back(color);
-
-	color.setNameRGB(		"black",		0.000,	0.000,	0.000);
+	color.setNameRGB("black",		0.000,	0.000,	0.000);
 	colorStack.push_back(color);
-
-	color.setNameRGB(		"red",			1.000,	0.000,	0.000);
+	color.setNameRGB("red",			1.000,	0.000,	0.000);
 	colorStack.push_back(color);
-
-	color.setNameRGB(		"brown",		0.647,	0.165,	0.165);
+	color.setNameRGB("brown",		0.647,	0.165,	0.165);
 	colorStack.push_back(color);
-
-	color.setNameRGB(		"tan",			0.824,	0.706,	0.549);
+	color.setNameRGB("tan",			0.824,	0.706,	0.549);
 	colorStack.push_back(color);
-
-	color.setNameRGB(		"orange",		1.000,	0.647,	0.000);
+	color.setNameRGB("orange",		1.000,	0.647,	0.000);
 	colorStack.push_back(color);
-
-	color.setNameRGB(		"yellow",		1.000,	1.000,	0.000);
+	color.setNameRGB("yellow",		1.000,	1.000,	0.000);
 	colorStack.push_back(color);
-
-	color.setNameRGB(		"green",		0.000,	1.000,	0.000);
+	color.setNameRGB("green",		0.000,	1.000,	0.000);
 	colorStack.push_back(color);
-
-	color.setNameRGB(		"ForestGreen",		0.133,	0.545,	0.133);
+	color.setNameRGB("ForestGreen",		0.133,	0.545,	0.133);
 	colorStack.push_back(color);
-
-	color.setNameRGB(		"cyan",			0.000,	1.000,	1.000);
+	color.setNameRGB("cyan",		0.000,	1.000,	1.000);
 	colorStack.push_back(color);
-
-	color.setNameRGB(		"blue",			0.000,	0.000,	1.000);
+	color.setNameRGB("blue",		0.000,	0.000,	1.000);
 	colorStack.push_back(color);
-
-	color.setNameRGB(		"skyblue",		0.529,	0.808,	0.922);
+	color.setNameRGB("skyblue",		0.529,	0.808,	0.922);
 	colorStack.push_back(color);
-
-	color.setNameRGB(		"magenta",		1.000,	0.000,	1.000);
+	color.setNameRGB("magenta",		1.000,	0.000,	1.000);
 	colorStack.push_back(color);
 }
 
@@ -310,9 +338,8 @@ display_colors()
 	return true;
 }
 
-// get full filename, searching through directory list in the environment
-// variable GRIINPUTS, or, if that environment variable doesn't exist, in the
-// symbol GRIINPUTS, defined in defaults.h
+#if 0
+// get full filename, searching through directory list
 char *
 file_in_list(const char *name, bool show_nonlocal_files, bool show_local_files)
 {
@@ -374,28 +401,32 @@ file_in_list(const char *name, bool show_nonlocal_files, bool show_local_files)
 #ifdef VMS
 		char tmp[1024];
 		sprintf(tmp, "DEL %s;*", gr_currentPSfilename());
-		//printf("Should system '%s'\n", tmp);
-		system(tmp);
+		call_the_OS(tmp, __FILE__, __LINE__);
 #else
 		sprintf(tmp, "rm -f %s", gr_currentPSfilename());
-		//printf("Should system '%s'\n", tmp);
-		system(tmp);
+		call_the_OS(tmp, __FILE__, __LINE__);
 #endif
 		gri_exit(1);
 		return NULL;		// never done, actually
 	}
 #endif
 }
+#endif
 
 static void
 get_input_simulation(int argc, char *argv[], int separator)
 {
-	int             i;
-	if (separator == argc)
+	if (separator >= argc)
 		return;
 	// Save the words following SEPARATOR into the stdin io buffer.
-	for (i = separator + 1; i < argc; i++)
+	extern std::vector<const char*> _gri_argv;
+	for (int i = separator; i < argc; i++) {
+		//printf("\t push %d <%s>\n",i,argv[i]);
+#if 0				// 2001-feb-23 vsn 2.6.0 (alpha)
 		gr_textsave(argv[i]);
+#endif
+		_gri_argv.push_back(argv[i]);
+	}
 }
 #undef SEPARATOR
 
@@ -413,15 +444,13 @@ set_defaults()
 	_colX.setName("x");
 	_colY.setName("y");
 	_colZ.setName("z");
-	_colR.setName("r");
-	_colTHETA.setName("theta");
 	_colWEIGHT.setName("weight");
 	_prompt.assign("gri: ");
 	PUT_VAR("..arrowsize..", ARROWSIZE_DEFAULT);
 	PUT_VAR("..exit_status..", 0.0);
 	_axesStyle = 0;
 	_braceLevel = 0;
-	_beep = false;
+	_gri_beep = false;
 	_chatty = 1;
 	// Fonts
 	PUT_VAR("..fontsize..", FONTSIZE_PT_DEFAULT);
@@ -489,22 +518,10 @@ create_builtin_synonyms()
 	char *            os;
 	char *            wd;
 	char *            home;
-#if defined(VMS) || defined(MSDOS) || defined(IS_DEC_ALPHA)
-	time_t          sec;
-#else
-#if defined(__DECCXX)
-	int             sec;
-#else
-	long            sec;
-#endif
-#endif
 	// Use this for return codes
 	if (!put_syn("\\.return_value.", "", true)) OUT_OF_MEMORY;
-	// \.awk. (name of awk program)
-	if (!put_syn("\\.awk.", AWK, true)) OUT_OF_MEMORY;
 	// \.version. (version number)
 	if (!put_syn("\\.version.", _gri_number, true)) OUT_OF_MEMORY;
-	//
 	// \.pid. (process ID)
 	sprintf(_grTempString, "%d", int(getpid()));
 	if (!put_syn("\\.pid.", _grTempString, true)) OUT_OF_MEMORY;
@@ -512,9 +529,10 @@ create_builtin_synonyms()
 	wd = pwd();
 	if (!put_syn("\\.wd.", wd, true)) OUT_OF_MEMORY;
 	// \.time. (time)
+	SECOND_TYPE sec;
 	time(&sec);
 	strcpy(_grTempString, asctime(localtime(&sec)));
-	_grTempString[strlen(_grTempString) - 1] = '\0';
+	_grTempString[-1 + strlen(_grTempString)] = '\0'; // trim newline
 	if (!put_syn("\\.time.", _grTempString, true)) OUT_OF_MEMORY;
 	//  \.user. (user name)
 	user = egetenv("USER");
@@ -685,9 +703,25 @@ interpret_optional_arguments(int argc, char *argv[])
 				} else if (!strcmp(argv[i], "-nowarn_offpage")){
 					_warn_offpage = false;
 				} else if (!strcmp(argv[i], "-directory")) {
+					user_gave_directory = true;
 					number_optional_arg++;
 					i++;
-					_lib_directory.assign(argv[i]);
+					if (i < argc)
+						_lib_directory.assign(argv[i]);
+					else {
+						err("`-directory' needs an argument.");
+						gri_exit(1);
+					}
+				} else if (!strcmp(argv[i], "-output")) {
+					number_optional_arg++;
+					i++;
+					if (i < argc) {
+						psname.assign(argv[i]);
+						gr_setup_ps_filename(psname.c_str());
+					} else {
+						err("`gri ... -output FILENAME' needs the FILENAME!");
+						gri_exit(1);
+					}
 				} else if (!strcmp(argv[i], "-directory_default")) {
 					gr_textput(_lib_directory.c_str());
 					gr_textput("\n");
@@ -712,13 +746,16 @@ interpret_optional_arguments(int argc, char *argv[])
 					}
 				} else if (!strcmp(argv[i], "-no_expecting")) {
 					warning("Gri no longer demands to find an `expecting' command, so the `-no_expecting' option can be dropped.");
-				} else if (!strncmp(argv[i], "-s", 2)) {
-					if (1 == sscanf(argv[i], "-s%d", &val)) {
-						PUT_VAR("..superuser..", ((double) val));
-					} else {
-						PUT_VAR("..superuser..", ((double) 1.0));
-					}
 				} else if (!strncmp(argv[i], "-superuser", 10)) {
+					if ('?' == *(argv[i] + 10)) {
+						superuser_flag *sf = sflag;
+						printf("Superuser flags:\n");
+						while (sf->action != NULL) {
+							printf("-superuser%d -- %s\n", sf->code, sf->action);
+							sf++;
+						}
+						gri_exit(0);
+					}
 					if (1 == sscanf(argv[i], "-superuser%d", &val)) {
 						PUT_VAR("..superuser..", ((double) val));
 					} else {
@@ -757,82 +794,91 @@ interpret_optional_arguments(int argc, char *argv[])
 void
 give_help()
 {
-	gr_textput("NAME\n");
-	gr_textput("     gri - draw scientific graphs\n");
-	gr_textput("\n");
-	gr_textput("SYNOPSIS\n");
-	gr_textput("     gri [OPTIONS] [command_file [postscript_file]]\n");
-	gr_textput("\n");
-	gr_textput("DESCRIPTION\n");
-	gr_textput("     If a command file (command_file) is named, commands are read from that file;\n");
-	gr_textput("     otherwise they are read from the keyboard.  If a command_file is named, then\n");
-	gr_textput("     a file in which to store the PostScript output may also be named; otherwise\n");
-	gr_textput("     it is stored in a file named gr-00.ps (or gr-01.ps if gr-00.ps exists, etc).\n");
-	gr_textput("\n");
-	gr_textput("     There are 3 special forms that do no graphing:\n");
-	gr_textput("       `gri -creator postscript_file'\n");
-	gr_textput("         Extracts the Gri commands that created the Gri PostScript file.\n");
-	gr_textput("       `gri -help' or `gri -h'\n");
-	gr_textput("         Prints this help message.\n");
-	gr_textput("       `gri -version' or `gri -v'\n");
-	gr_textput("         Prints the version number of Gri.\n");
-	gr_textput("\n");
-	gr_textput("     In normal usage, where drawing is expected, Gri takes these options:\n");
-	gr_textput("     -batch or -b\n");
-	gr_textput("             Stops printing of prompts and hints.\n");
-	gr_textput("     -chatty[N] or -c[N]\n");
-	gr_textput("             Let gri print info messages\n");
-	gr_textput("     -debug or -d\n");
-	gr_textput("             Turns debugging on (sets variable ..debug.. to value 1).\n");
-	gr_textput("     -warn_offpage\n");
-	gr_textput("             Warn if any item is drawn far off a 8.5x11\" page.\n");
-	gr_textput("             (This is the default.)\n");
-	gr_textput("     -nowarn_offpage\n");
-	gr_textput("             Don't warn if any item is drawn far off a 8.5x11\" page\n");
+        gr_textput("NAME\n");
+        gr_textput("     gri - draw scientific graphs\n");
+        gr_textput("\n");
+        gr_textput("SYNOPSIS\n");
+        gr_textput("     gri [OPTIONS] [command_file [postscript_file]]\n");
+        gr_textput("\n");
+        gr_textput("DESCRIPTION\n");
+        gr_textput("     If a command file (command_file) is named, commands are read from that file;\n");
+        gr_textput("     otherwise they are read from the keyboard.  If a command_file is named, then\n");
+        gr_textput("     a file in which to store the PostScript output may also be named; otherwise\n");
+        gr_textput("     it is stored in a file named gr-00.ps (or gr-01.ps if gr-00.ps exists, etc).\n");
+        gr_textput("\n");
+        gr_textput("     There are 3 special forms that do no graphing:\n");
+        gr_textput("       `gri -creator postscript_file'\n");
+        gr_textput("         Extracts the Gri commands that created the Gri PostScript file.\n");
+        gr_textput("       `gri -help' or `gri -h'\n");
+        gr_textput("         Prints this help message.\n");
+        gr_textput("       `gri -version' or `gri -v'\n");
+        gr_textput("         Prints the version number of Gri.\n");
+        gr_textput("\n");
+        gr_textput("     In normal usage, where drawing is expected, Gri takes these options:\n");
+        gr_textput("     -batch or -b\n");
+        gr_textput("             Stops printing of prompts and hints.\n");
+        gr_textput("     -chatty[N] or -c[N]\n");
+        gr_textput("             Let gri print info messages\n");
+        gr_textput("     -debug or -d\n");
+        gr_textput("             Turns debugging on (sets variable ..debug.. to value 1).\n");
+        gr_textput("     -warn_offpage\n");
+        gr_textput("             Warn if any item is drawn far off a 8.5x11\" page.\n");
+        gr_textput("             (This is the default.)\n");
+        gr_textput("     -nowarn_offpage\n");
+        gr_textput("             Don't warn if any item is drawn far off a 8.5x11\" page\n");
 
-	gr_textput("     -directory pathname\n");
-	gr_textput("             Specifies the directory where Gri looks for startup files;\n");
-	gr_textput("             otherwise it looks in /opt/gri/lib or at whatever\n");
-	gr_textput("             directory is defined in configure shellscript, at compile time.\n");
-	gr_textput("     -directory_default\n");
-	gr_textput("             Reports directory where gri.cmd should be found,\n");
-	gr_textput("             if not supplied by -directory.\n");
-	gr_textput("     -no_bounding_box\n");
-	gr_textput("             Make bounding-box be full page.\n");
-	gr_textput("     -no_expecting\n");
-	gr_textput("             Prevent warning message if `expecting version .n.'\n");
-	gr_textput("             command is missing.\n");
-	gr_textput("     -no_startup_message\n");
-	gr_textput("             Stops printing of startup message.\n");
-	gr_textput("     -publication or -p\n");
-	gr_textput("             Sets the builtin variable ..publication.. to 1; normally it is 0.\n");
-	gr_textput("             One might use if statements (`if !..publication..' ...) on drafts.\n");
-	gr_textput("     -superuser or -s\n");
-	gr_textput("	     Used only by Gri programmers (who can check the value with the\n");
-	gr_textput("	     C function `superuser()'.)  An optional value can be supplied\n");
-	gr_textput("	     without spaces (e.g. `-s2') to set the debugging level.\n");
-	gr_textput("             Flags are listed below; add flags to get several actions at once\n");
-	gr_textput("	       1: print cmdline before/after substituting synonyms\n");
-	gr_textput("	       2: print cmdline before/after substituting rpn expressions\n");
-	gr_textput("	       4: print new commands being defined\n");
-	gr_textput("	       8: print system commands and `open \"... | \"' commands before\n");
-	gr_textput("	          they are passed to the system\n");
-	gr_textput("	     128: for author's use only\n");
-	gr_textput("	     256: for author's use only\n");
-	gr_textput("	      Note that all flags are equal\n");
-	gr_textput("	      to 2 raised to an integer power.  Since the flag values are\n");
-	gr_textput("	      detected by a bitwise OR, you can combine flags by adding; thus\n");
-	gr_textput("	      specifying a flag of 5 yields flags 1 and 4 together; specifying\n");
-	gr_textput("	      15 yields flags 1, 2, 4 and 8.\n");
-	gr_textput("     -trace or -t\n");
-	gr_textput("             Makes Gri print out command lines as they are executed.\n");
-	gr_textput("     -true or -y\n");
-	gr_textput("             Makes Gri think the answer to all `query's is RETURN.\n");
+        gr_textput("     -directory pathname\n");
+        gr_textput("             Specifies the directory where Gri looks for startup files;\n");
+        gr_textput("             otherwise it looks in /opt/gri/lib or at whatever\n");
+        gr_textput("             directory is defined in configure shellscript, at compile time.\n");
+        gr_textput("     -directory_default\n");
+        gr_textput("             Reports directory where gri.cmd should be found,\n");
+        gr_textput("             if not supplied by -directory.\n");
+
+
+        gr_textput("     -no_bounding_box\n");
+        gr_textput("             Make bounding-box be full page.\n");
+        gr_textput("     -no_expecting\n");
+        gr_textput("             Prevent warning message if `expecting version .n.'\n");
+        gr_textput("             command is missing.\n");
+        gr_textput("     -no_startup_message\n");
+        gr_textput("             Stops printing of startup message.\n");
+        gr_textput("     -output PS_file_name\n");
+        gr_textput("             Specify the PostScript filename.  If this\n");
+        gr_textput("             is not specified, the PostScript filename is derived\n");
+        gr_textput("             from the name of the commandfile (e.g. `mygraph.gri'\n");
+        gr_textput("              produces `mygraph.ps'), or, for interactive use,\n");
+        gr_textput("             it will have a name like `gri-00.ps', or\n");
+        gr_textput("             `gri-01.ps' if the former file exists, etc.\n");
+        gr_textput("     -publication or -p\n");
+        gr_textput("             Sets the builtin variable ..publication.. to 1; normally it is 0.\n");
+        gr_textput("             One might use if statements (`if !..publication..' ...) on drafts.\n");
+        gr_textput("     -superuser\n");
+        gr_textput("             Used mainly by Gri programmers (who can check the value with the\n");
+        gr_textput("             C function `superuser()'.)  An optional value can be supplied\n");
+        gr_textput("             without spaces (e.g. `-s2') to set the debugging level.\n");
+        gr_textput("             The flags are as follows:\n");
+        gr_textput("                 1: print cmdline before/after substituting synonyms\n");
+        gr_textput("                 2: print cmdline before/after substituting rpn expressions\n");
+        gr_textput("                 4: print new commands being defined\n");
+        gr_textput("                 8: print system commands and `open \"... | \"'\n");
+        gr_textput("                    commands before they are passed to the system\n");
+        gr_textput("               128: for author's use only\n");
+        gr_textput("               256: for author's use only\n");
+        gr_textput("             Note that all flags are equal to 2 raised to an\n");
+        gr_textput("             integer power.  Since the flag values are detected by\n");
+        gr_textput("             a bitwise OR, you can combine flags by adding; thus\n");
+        gr_textput("             specifying a flag of 5 yields flags 1 and 4 together; specifying\n");
+        gr_textput("             15 yields flags 1, 2, 4 and 8.\n");
+        gr_textput("     -trace or -t\n");
+        gr_textput("             Makes Gri print out command lines as they are executed.\n");
+        gr_textput("     -true or -y\n");
+        gr_textput("             Makes Gri think the answer to all `query's is RETURN.\n");
 #if 0
-	gr_textput("     -e cmd\n");
-	gr_textput("             BUG: NOT IMPLEMENTED YET!\n");
-	gr_textput("             Makes Gri perform indicated command after\n");
-	gr_textput("             doing any commands in the ~/.grirc file.\n");
+        gr_textput("     -e cmd\n");
+        gr_textput("             BUG: NOT IMPLEMENTED YET!\n");
+        gr_textput("             Makes Gri perform indicated command after\n");
+        gr_textput("             doing any commands in the ~/.grirc file.\n");
 #endif
 }
 
@@ -840,7 +886,7 @@ give_help()
 
 // Insert Creator info in PS file
 static void
-insert_creator_name_in_PS(int argc, char *argv[], const string& psname)
+insert_creator_name_in_PS(int argc, char *argv[], const std::string& psname)
 {
 	extern char _gri_release_time[];
 	extern char _gri_number[]; // see version.c
@@ -868,7 +914,6 @@ insert_creator_name_in_PS(int argc, char *argv[], const string& psname)
 		// No commandfile supplied on cmdline.
 		Require2(put_syn("\\.command_file.", "stdin", true), OUT_OF_MEMORY);
 		Require2(put_syn("\\.readfrom_file.", "stdin", true), OUT_OF_MEMORY);
-		Require2(put_syn("\\.ps_file.", "gri-??.ps", true), OUT_OF_MEMORY);
 	}
 }
 
@@ -889,12 +934,14 @@ dogrirc()
 	strcpy(grircname, GRIRC_FILE);
 #endif
 	if (push_cmd_file(grircname, false, false, "r")) {
-		gr_comment("gri:// BEGIN (~/.grirc)\n");
+		gr_comment("gri:#\n");
+		gr_comment("gri:# The user's ~/.grirc file ...\n");
 		while (do_command_line()){
 			;
 		}
 		_done = 0;
-		gr_comment("gri:// END   (~/.grirc)\n");
+		gr_comment("gri:# ... end of users ~/.grirc file.\n");
+		gr_comment("gri:\n");
 	}
 }
 
@@ -902,7 +949,7 @@ dogrirc()
 static void
 show_startup_msg()
 {
-	string fullfilename(_lib_directory.c_str());
+	std::string fullfilename(_lib_directory.c_str());
 	// Must check for '/' as file separator, on some machines.
 #if !defined(VMS)
 #if defined(MSDOS)
