@@ -1,10 +1,15 @@
 //#define DEBUG_RE
+//#define DEBUG_UNDERSCORE 
 
-#include	<string>
-#include	<ctype.h>
-#include	<math.h>
-#include	<stdio.h>
-#include        <stdarg.h>
+#include <string>
+#include <ctype.h>
+#include <math.h>
+#include <stdio.h>
+#include <stdarg.h>
+#if !defined(IS_MINGW32)
+#include <pwd.h>
+#endif
+#include <sys/types.h>
 
 #if defined(MSDOS)		// need all these?
 #include <stdlib.h>
@@ -26,10 +31,295 @@
 static void     show_pattern(char *target, int tlen, int star, int plus);
 #endif
 extern double   strtod(const char *, char **);
+
+bool
+get_normal_number(const char *s, double *d)
+{
+	char *ptr = NULL;
+	*d = strtod(s, &ptr);
+	if (*ptr == '\0') {
+		// Normal number; check for infinity/not-a-number
+#if defined(HAVE_ISNAN) && defined(HAVE_ISINF)
+#if !defined(__MACHTEN__)
+		extern double _grMissingValue;
+		if (isinf(*d) || isnan(*d))
+			*d = (double) _grMissingValue;
+#endif
+#endif
+		return true;
+	} else {
+		return false;
+	}
+}
+
+// This warning business is too hard.  Just ignore it for now.
+#if 1
+bool
+get_number_with_underscores(const char *s, double *value)
+{
+	const char *si = s;
+	static std::string ss;	// keep in storage 
+	ss.assign("");
+	unsigned int slen = strlen(s);
+	if (!isdigit(*si) && *si != '.' && *si != '+' && *si != '-')
+		return false;
+	if (*si == '.' && !isdigit(*(si + 1)))
+		return false;
+	for (unsigned int i = 0; i < slen; i++) {
+		if (isdigit(*si) || *si == '.' || *si == '-' || *si == '+'
+		    || *si == 'e' || *si == 'E' || *si == 'd' || *si == 'D') {
+			ss += *si;
+		} else if (*si == '_') {
+			;
+		} else {
+			return false;
+		}
+		si++;
+	}
+	if (get_normal_number(ss.c_str(), value))
+		return true;
+	else
+		return false;
+}
+#else
+bool
+get_number_with_underscores(const char *s, double *value)
+{
+	const char *si = s;
+	static std::string ss;	// keep in storage 
+	ss.assign("");
+	unsigned int slen = strlen(s);
+	unsigned int last_underline = 0;
+	bool have_underline = false;
+	bool have_decimal = false;
+	if (!isdigit(*si) && *si != '.' && *si != '+' && *si != '-')
+		return false;
+	if (*si == '.' && !isdigit(*(si + 1)))
+		return false;
+	for (unsigned int i = 0; i < slen; i++) {
+		if (isdigit(*si) || *si == '-' || *si == '+') {
+			ss += *si;
+		} else if (*si == '.') {
+			ss += *si;
+			if (have_underline && ((i - 1 - last_underline) != 3))
+				warning("misplaced _ before decimal place in `\\", s, "'", "\\");
+			last_underline = i; // pretend
+			have_underline = false;
+			have_decimal = true;
+		} else if (*si == 'e' || *si == 'E' || *si == 'd' || *si == 'D') {
+			ss += *si;
+			if (have_underline && ((i - 1 - last_underline) != 3))
+				warning("misplaced _ before exponent indicator in `\\", s, "'", "\\");
+			last_underline = i;
+			have_underline = false;	// reset
+		} else if (*si == '_') {
+			if (have_underline && ((i -1 - last_underline) != 3))
+				warning("misplaced _ in numerical constant `\\", s, "'", "\\");
+			last_underline = i;
+			have_underline = true;
+		} else {
+#ifdef DEBUG_UNDERSCORE
+			printf("%s:%d NOT A NUMBER '%s'\n", __FILE__,__LINE__,s);
+#endif
+			return false;
+		}
+		si++;
+	}
+#ifdef DEBUG_UNDERSCORE
+	printf("%s:%d last_underscore %d    end at %d\n", __FILE__,__LINE__,last_underline, slen);
+#endif
+	if (have_underline && (slen - 1 - last_underline != 3)) {
+		warning("misplaced _ in constant `\\", s, "'", "\\");
+	}
+#ifdef DEBUG_UNDERSCORE
+	printf("%s:%d OK, translated '%s' -> '%s'\n", __FILE__,__LINE__,s,ss.c_str());
+#endif
+	if (get_normal_number(ss.c_str(), value)) {
+#ifdef DEBUG_UNDERSCORE
+		printf("%s:%d Right this is a number '%s' -> '%s' -> %e\n", __FILE__,__LINE__,s,ss.c_str(), *value);
+#endif
+		return true;
+	}
+#ifdef DEBUG_UNDERSCORE
+	printf("%s:%d Not a number '%s' -> '%s'\n", __FILE__,__LINE__,s,ss.c_str());
+#endif
+	return false;
+}
+#endif
+
+bool
+get_coded_value(const std::string& name, int level, std::string& result)
+{
+	//printf("DEBUG %s:%d get_coded_value(<%s>,%d)\n",__FILE__,__LINE__,name.c_str(),level);
+	int mark_above = level + 1;
+	if (name[0] == '.') {
+		int index = index_of_variable(name.c_str(), level);
+		//printf("DEBUG %s:%d index %d\n",__FILE__,__LINE__,index);
+		if (index < 0) {
+			// No such variable known
+			return false;
+		}
+		char buf[100]; // BUG: could be too short
+		sprintf(buf, "%g", variableStack[index].get_value());
+		result.assign(buf);
+		return true;
+	} else if (name[0] == '\\'){
+		int mark = 0;
+		int index;
+		for (index = 0; index < int(synonymStack.size()); index++) {
+			const char *n = synonymStack[index].get_name();
+			if (*n == '\0')
+				if (++mark == mark_above)
+					break;
+		}
+		if (mark != mark_above) {
+			//printf("DEBUG %s:%d no match for <%s>\n",__FILE__,__LINE__,name.c_str());
+			return false;
+		}
+		//printf("DEBUG %s:%d index %d\n",__FILE__,__LINE__,index);
+		for (int i = index - 1; i >= 0; i--) {
+			if (synonymStack[i].get_name() == name) {
+				//printf("DEBUG %s:%d match at i= %d\n",__FILE__,__LINE__,i);
+				result.assign(synonymStack[i].get_value());
+				return true;
+			}
+		}
+	} else {
+		err("Internal error in synonyms.cc; cannot decode `\\", name.c_str(), "'", "\\");
+		return false;
+	}
+        return true;
+}
+
+bool 
+is_coded_string(const std::string&s, std::string& name, int* mark_level)
+{
+	//printf("DEBUG %s:%d is_coded_string <%s>\n",__FILE__,__LINE__,s.c_str());
+
+	//for (unsigned int i = 0; i < s.size(); i++) printf("\ts[%d] = '%c'\n",i,s[i]);
+
+	if (s.size() < 33) { // this long even if var and level each have only 1 character
+		return false;
+	}
+	char cname[100];	// BUG: may be too short
+	int ml = 0;
+	if (2 != sscanf(s.c_str(), AMPERSAND_CODING, cname, &ml)) {
+		//printf("DEBUG: %s:%d problem ... cname [%s]  level %d\n",cname, ml);
+		return false;
+	}
+	name.assign(cname);
+	*mark_level = ml;
+	//printf("DEBUG %s:%d decoded name <%s> level %d\n",__FILE__,__LINE__,name.c_str(),*mark_level);
+	return true;
+}
+
+
+bool
+marker_draw() // put a marker on top
+{
+	GriVariable markVar("", 0.0);
+	variableStack.push_back(markVar);
+	GriSynonym markSyn("", "");
+	synonymStack.push_back(markSyn);
+	return true;
+}
+int
+marker_count() // -1 if error
+{
+	int nv = 0;
+	int vlen = variableStack.size();
+	for (int i = vlen - 1; i >= 0; i--)
+		if (*variableStack[i].get_name() == '\0')
+			nv++;
+
+	int ns = 0;
+	int slen = synonymStack.size();
+	for (int ii = slen - 1; ii >= 0; ii--)
+		if (*synonymStack[ii].get_name() == '\0')
+			ns++;
+	if (ns == nv)
+		return ns;
+	return -1;		// error
+}
+bool
+marker_erase() // erase top marker; return false if there is none
+{
+	bool ok = false;
+	int vlen = variableStack.size();
+	for (int i = vlen - 1; i >= 0; i--) {
+		if (*variableStack[i].get_name() == '\0') {
+			variableStack.erase(variableStack.begin() + i);
+			ok = true;
+			break;
+		}
+	}
+	if (ok) {
+		ok = false;
+		int slen = synonymStack.size();
+		for (int i = slen - 1; i >= 0; i--) {
+			if (*synonymStack[i].get_name() == '\0') {
+				synonymStack.erase(synonymStack.begin() + i);
+				ok = true;
+				break;
+			}
+		}
+	}
+	return ok;
+}
 
 // Keep this in static area, so as not to waste time with
 // multiple allocation/deallocation for scratch strings
-static string tmp_string;
+static std::string tmp_string;
+
+bool
+get_nth_word(const std::string& s, unsigned int which, std::string& result)
+{
+	char *w[MAX_nword];	// BUG: wasteful
+	unsigned int nw;
+	char *cpy = strdup(s.c_str());
+	chop_into_words(cpy, w, &nw, MAX_nword);
+	if (nw <= which)
+		return false;
+	result.assign(w[which]);
+	free(cpy);
+	return true;
+}
+
+unsigned int
+get_number_of_words(const std::string& s)
+{
+	if (s.size() == 0)
+		return 0;
+	char *w[MAX_nword];	// BUG: wasteful
+	unsigned int nw;
+	char *cpy = strdup(s.c_str());
+	chop_into_words(cpy, w, &nw, MAX_nword);
+	free(cpy);
+	return nw;
+
+}
+
+
+bool
+is_assignment_op(const char *s)
+{
+	if (!strcmp(s, "="))
+		return true;
+	if (!strcmp(s, "+="))
+		return true;
+	if (!strcmp(s, "*="))
+		return true;
+	if (!strcmp(s, "-="))
+		return true;
+	if (!strcmp(s, "/="))
+		return true;
+	if (!strcmp(s, "^="))
+		return true;
+	if (!strcmp(s, "_="))
+		return true;
+	return false;
+}
+
 
 // Return true if string is all space/tab chars
 bool
@@ -48,21 +338,19 @@ number_missing_cols()
 {
 	// Only examine other columns if they have same length
 	unsigned int length = _colX.size();
-	bool do_y     = _colY.size()     == length;
-	bool do_z     = _colZ.size()     == length;
-	bool do_U     = _colU.size()     == length;
-	bool do_V     = _colV.size()     == length;
-	bool do_R     = _colR.size()     == length;
-	bool do_THETA = _colTHETA.size() == length;
+	bool do_y      = _colY.size()      == length;
+	bool do_z      = _colZ.size()      == length;
+	bool do_U      = _colU.size()      == length;
+	bool do_V      = _colV.size()      == length;
+	bool do_WEIGHT = _colWEIGHT.size() == length;
 	unsigned int missing = 0;
 	for (unsigned int i = 0; i < length; i++) {
-		if (            gr_missing(_colX[i]))     { missing++; continue; }
-		if (do_y     && gr_missing(_colY[i]))	  { missing++; continue; }
-		if (do_z     && gr_missing(_colZ[i]))	  { missing++; continue; }
-		if (do_U     && gr_missing(_colU[i]))	  { missing++; continue; }
-		if (do_V     && gr_missing(_colV[i]))	  { missing++; continue; }
-		if (do_R     && gr_missing(_colR[i]))	  { missing++; continue; }
-		if (do_THETA && gr_missing(_colTHETA[i])) { missing++; continue; }
+		if (             gr_missing(_colX[i]))      { missing++; continue; }
+		if (do_y      && gr_missing(_colY[i]))	    { missing++; continue; }
+		if (do_z      && gr_missing(_colZ[i]))	    { missing++; continue; }
+		if (do_U      && gr_missing(_colU[i]))	    { missing++; continue; }
+		if (do_V      && gr_missing(_colV[i]))	    { missing++; continue; }
+		if (do_WEIGHT && gr_missing(_colWEIGHT[i])) { missing++; continue; }
 	}
 	return missing;
 }
@@ -114,7 +402,7 @@ remove_esc_quotes(char *w)
 bool
 word_is(int i, const char *word)
 {
-	return ((-1 < i && i < _nword && !strcmp(word, _word[i])) ? true : false);
+	return ((-1 < i && i < int(_nword) && !strcmp(word, _word[i])) ? true : false);
 }
 
 void
@@ -130,64 +418,22 @@ check_psfile()
 bool
 delete_file(const char *filename)
 {
+#if defined(HAVE_UNISTD_H)
+	return !(unlink(filename));
+#else  // will have to delete it 'manually'
 	char            sys_cmd[200];
 #if defined(VMS)
 	sprintf(sys_cmd, "DEL %s;*", filename);
-	system(sys_cmd);
-	return true;
+	call_the_OS(sys_cmd, __FILE__, __LINE__);
 #elif defined(MSDOS)
 	sprintf(sys_cmd, "DEL %s", filename);
-	system(sys_cmd);
-	return true;
+	call_the_OS(sys_cmd, __FILE__, __LINE__);
 #else
 	sprintf(sys_cmd, "rm %s", filename);
-	system(sys_cmd);
+	call_the_OS(sys_cmd, __FILE__, __LINE__);
+#endif
+#endif // whether have unlink
 	return true;
-#endif
-}
-
-// Convert full filename to tilde name
-char *
-cshell_name(char *s)
-{
-	char *            new_name;
-#if defined(HAVE_GETENV)
-#if defined(MSDOS)		// for MSDOS, return input
-	new_name = new char [1 + strlen(s)];
-	if (!new_name) OUT_OF_MEMORY;
-	strcpy(new_name, s);
-	return new_name;
-#else				// not MSDOS
-	char *            user = (char *) getenv("USER");
-	char *            home = (char *) getenv("HOME");
-	if (user && home) {
-		// The environment variables were recovered ok.
-		if (!strncmp(s, home, strlen(home))) {
-			// Below asks for 1 extra space; what the heck.
-			new_name = new char [3 + strlen(user) + strlen(s) - strlen(home)];
-			if (!new_name) OUT_OF_MEMORY;
-			strcpy(new_name, "~");
-			strcat(new_name, user);
-			strcat(new_name, s + strlen(home));
-		} else {
-			new_name = new char[1 + strlen(s)];
-			if (!new_name) OUT_OF_MEMORY;
-			strcpy(new_name, s);
-		}
-	} else {
-		// Could not recover environment variables
-		new_name = new char[1 + strlen(s)];
-		if (!new_name) OUT_OF_MEMORY;
-		strcpy(new_name, s);
-	}
-	return new_name;
-#endif				// not MSDOS
-#else				// No environment variables
-	new_name = new char[1 + strlen(s)];
-	if (!new_name) OUT_OF_MEMORY;
-	strcpy(new_name, s);
-	return new_name;
-#endif
 }
 
 /*
@@ -214,38 +460,43 @@ skip_nonspace(const char *s)
 	return i;
 }
 
-// Store result of 'printenv _word[3]' into _word[2] (If no such environment
-// variable, store "" into _word[2].)
+// 'get env \a SHELL'
 bool
 get_envCmd()
 {
-	if (_nword == 4) {
-		char *            result;
-		if (!is_syn(_word[2])) {
-			err("No synonym name given");
-			demonstrate_command_usage();
-			return false;
-		}
-		result = egetenv((const char*)_word[3]);
-		if (result == NULL) {
-			if (!put_syn(_word[2], "", true)) {
-				gr_Error("Ran out of storage");
-				return false;
-			}
-			warning("No environment variable called `\\", _word[3], "' exits.", "\\");
-		} else {
-			if (!put_syn(_word[2], result, true)) {
-				gr_Error("Ran out of storage");
-				return false;
-			}
-		}
-	} else {
+	if (_nword != 4) {
 		NUMBER_WORDS_ERROR;
 		demonstrate_command_usage();
 		return false;
 	}
-	return true;			/* never reached */
+	std::string the_syn(_word[2]);
+	un_double_quote(the_syn);
+	un_double_slash(the_syn);
+	de_reference(the_syn);
+	if (!is_syn(the_syn)) {
+		err("No synonym name given");
+		demonstrate_command_usage();
+		return false;
+	}
+	std::string the_env_var(_word[3]);
+	un_double_quote(the_env_var);
+	de_reference(the_env_var);
+	char *result = egetenv(the_env_var.c_str());
+	if (result == NULL) {
+		if (!put_syn(the_syn.c_str(), "", true)) {
+			gr_Error("Ran out of storage");
+			return false;
+		}
+		warning("No environment variable called `\\", the_env_var.c_str(), "' exits.", "\\");
+	} else {
+		if (!put_syn(the_syn.c_str(), result, true)) {
+			gr_Error("Ran out of storage");
+			return false;
+		}
+	}
+	return true;
 }
+
 
 /* Paste a single character onto the end of a string. */
 void
@@ -283,84 +534,125 @@ is_punctuation(int c)
 		) ? true : false);
 }
 
-char *
-complete_filename(const char *filename)
+bool
+resolve_filename(std::string& f, bool trace_path, char c_or_d)
 {
-	char *            return_value;
-#if defined(VMS)
-	return_value = strdup(filename);
-	return return_value;
-#elif defined(MSDOS)
-	return_value = strdup(filename);
-	return return_value;
-#else
-	string rval;
-	if (filename[0] == '~') {
-		if (strlen(filename) < 2)
-			return NULL; // broken filename
-		rval.append(egetenv("HOME"));
-		rval.append(filename + 1);
-		return_value = new char[1 + rval.size()];
-		if (!return_value) OUT_OF_MEMORY;
-		strcpy(return_value, rval.c_str());
-	} else if (filename[0] == '/' || filename[0] == '\\') {
-		return_value = new char[1 + strlen(filename)];
-		if (!return_value) OUT_OF_MEMORY;
-		strcpy(return_value, filename);
-	} else {
-		// Normal file.  Prepend current working directory
-		return_value = new char[2
-				       + strlen(_current_directory.c_str())
-				       + strlen(filename)];
-		if (!return_value) OUT_OF_MEMORY;
-		strcpy(return_value, _current_directory.c_str());
-#if defined(MSDOS)
-		strcat(return_value, "\\");
-#else
-		strcat(return_value, "/");
-#endif
-		strcat(return_value, filename);
+	unsigned int len;
+	if (f[0] == '"') {
+		f.STRINGERASE(0,1);
+		len = f.size();
+		if (len < 2)
+			return false;
+		if (f[len - 1] != '"')
+			return false;
+		f.STRINGERASE(len-1, 1);
 	}
-	remove_trailing_blanks(return_value);
-	return return_value;
-#endif
-}
-
-char *
-tilde_expand(const char *filename)
-{
-	char *return_value;
-	if (filename[0] != '~') {
-		return_value = new char[1 + strlen(filename)];
-		if (!return_value) OUT_OF_MEMORY;
-		strcpy(return_value, filename);
-	} else {
-		char *home = egetenv("HOME");
-		return_value = new char[1 + strlen(filename) - 2 + strlen(home)];
-		if (!return_value) OUT_OF_MEMORY;
-		strcpy(return_value, home);
-		strcat(return_value, "/");
-		strcat(return_value, filename + 2); // skip the ~/ chars
+	// Change any escaped quotes to quotes (not sure why
+	// anybody would do this, but what the heck).
+	unsigned int i;
+	len = f.size();
+	for (i = 1; i < len; i++) {
+		if (f[i] == '"' && f[i-1] == '\\') {
+			f.STRINGERASE(i - 1, 1);
+			if (--len == 0)
+				return false;
+		}
 	}
-	return return_value;
+	if (f[0] == '~') {
+		if (f[1] == '/') {
+			f.STRINGERASE(0, 1);
+			f.insert(0, egetenv("HOME"));
+			return true;
+		} else {
+#if !defined(IS_MINGW32)
+			size_t name_end = f.find("/");
+			if (name_end == STRING_NPOS)
+				name_end = f.size();
+			std::string username = f.substr(1, name_end - 1);
+			struct passwd *pw_entry;
+			pw_entry = getpwnam(username.c_str());
+			f.STRINGERASE(0, username.size() + 1);
+			f.insert(0, pw_entry->pw_dir);
+			return true;
+#else
+			return false;
+#endif
+		}
+	}
+
+	// BUG: probably should substitute any env-vars here, e.g. $HOME
+
+	// Done, unless we have to trace the path ...
+	if (!trace_path)
+		return true;
+
+	// ... but can even skip that, if the pathname is complete already ...
+	if (f[0] == '.' || f[0] == '/')
+		return true;
+
+	// ... ok, now we know we should trace!
+	std::string path;
+	if (c_or_d == 'c') {
+		if (!get_syn("\\.path_commands.", path)) {
+			err("Internal error in utility.cc:resolve_filename() -- cannot determine value of \\.path_commands\n");
+			return false;
+		}
+	} else if (c_or_d == 'd') {
+		if (!get_syn("\\.path_data.", path)) {
+			err("Internal error in utility.cc:resolve_filename() -- cannot determine value of \\.path_data\n");
+			return false;
+		}
+	} else {
+		err("Internal error in utility.cc:resolve_filename() -- c_or_d has unacceptable value\n");
+		return false;
+	}
+	if (path.size() < 1)
+		return true;	// BUG: is this what I want for empty path?
+	// HAD: path.assign(GRIINPUTS); // in defaults.hh ".:/usr/local/lib/gri"
+	std::string::size_type start = 0;
+	std::string::size_type colon;
+	do {
+		colon = path.find(":", start);
+		std::string test_file = path.substr(start, colon);
+		test_file.append("/");
+		test_file.append(f);
+		//printf("resolve_filename trying file named '%s'\n", test_file.c_str());
+		FILE *fp = fopen(test_file.c_str(), "r");
+		if (fp != NULL) {
+			fclose(fp);
+			f = test_file;
+			return true;
+		}
+		start = colon + 1; // skip the ':'
+	} while (colon != STRING_NPOS);
+
+	// Well, we just can't find this file.  Too bad.
+	return false;
 }
+
 
+
 char *
 pwd()
 {
-#if defined(VMS)		/* vms version braindead */
+#if HAVE_GETCWD
+	static char msg[1024];
+	if (NULL == getcwd(msg, 1024))
+		return "";
+	else
+		return msg;
+#elif defined(VMS)		/* vms version braindead */
 	return "";
 #elif defined(MSDOS)		/* msdos version braindead */
 	return "";
-#else
-#if !defined(HAVE_POPEN)
-	err("Cannot do `pwd' because computer lacks popen() subroutine");
+#elif !defined(HAVE_POPEN)
+	//err("Cannot do `pwd' because computer lacks popen() subroutine");
 	return "";
 #else
 	char            msg[1024];
 	FILE           *pipefile = (FILE *) popen("pwd", "r");
 	if (pipefile) {
-		char *            result;
+		char *result;
 		if (1 == fscanf(pipefile, "%s", msg)) {
 			pclose(pipefile);
 			result = new char[1 + strlen(msg)];
@@ -379,17 +671,23 @@ pwd()
 		return ".";
 	}
 #endif
-#endif
 }
 
-char *
+char*
 egetenv(const char *s)
 {
+#if defined(HAVE_GETENV)
+	char *rval = "";
+#endif
 	if (!strcmp(s, "PWD")) {
 		return (char *) pwd();
 	} else if (!strcmp(s, "USER")) {
 #if defined(HAVE_GETENV)
-		return (char *) getenv(s);
+		rval = (char *)getenv(s);
+		if (rval == NULL)
+			return "";
+		else
+			return rval;
 #else
 		return "unknown";
 #endif
@@ -403,34 +701,51 @@ egetenv(const char *s)
 #endif
 	} else if (!strcmp(s, "HOST")) {
 #if defined(HAVE_GETENV)
-		return (char *) getenv(s);
+		rval = (char *)getenv(s);
+		if (rval == NULL)
+			return "";
+		else
+			return rval;
 #else
 		return "unknown";
 #endif
 	} else if (!strcmp(s, "HOME")) {
 #if defined(HAVE_GETENV)
-		return (char *) getenv(s);
+		rval = (char *)getenv(s);
+		if (rval == NULL)
+			return "";
+		else
+			return rval;
 #else
 		return "unknown";
 #endif
 	} else if (!strcmp(s, "PAGER")) {
 #if defined(HAVE_GETENV)
-		return (char *) getenv(s);
+		rval = (char *)getenv(s);
+		if (rval == NULL)
+			return "";
+		else
+			return rval;
 #else
 		return "unknown";
 #endif
 	} else if (!strcmp(s, "GRIINPUTS")) {
-		/*
-		 * Of form ".:/usr/local/lib/gri"
-		 */
 #if defined(HAVE_GETENV)
-		return (char *) getenv(s);
+		rval = (char *)getenv(s);
+		if (rval == NULL)
+			return "";
+		else
+			return rval;
 #else
-		return NULL;		/* will use default */
+		return "";
 #endif
 	} else {
 #if defined(HAVE_GETENV)
-		return (char *) getenv(s);
+		rval = (char *)getenv(s);
+		if (rval == NULL)
+			return "";
+		else
+			return rval;
 #else
 		return "unknown";
 #endif
@@ -442,8 +757,7 @@ egetenv(const char *s)
 bool
 sprintfCmd()
 {
-	int             i;
-	char *            fmt;
+	char           *fmt;
 	char            msg[1024];
 	double          x[N];
 	if (_nword < 4) {
@@ -452,7 +766,7 @@ sprintfCmd()
 	}
 	*(_word[2] + strlen(_word[2]) - 1) = '\0';
 	fmt = _word[2] + 1;
-	for (i = 3; i < _nword; i++) {
+	for (unsigned int i = 3; i < _nword; i++) {
 		getdnum(_word[i], &x[i - 3]);
 	}
 	switch (_nword) {
@@ -570,12 +884,14 @@ sprintfCmd()
 		err("Can only do `sprintf' for 1-20 variables");
 		return false;
 	}
-	if (!is_syn(_word[1])) {
+	std::string w1(_word[1]); 
+	de_reference(w1);
+	if (!is_syn(w1)) {
 		demonstrate_command_usage();
-		err("Second word of command must begin with `\\'");
+		err("Second word of command must be a synonym-name'");
 		return false;
 	}
-	if (!put_syn(_word[1], msg, true)) {
+	if (!put_syn(w1.c_str(), msg, true)) {
 		gr_Error("Ran out of storage");
 		return false;
 	}
@@ -627,7 +943,7 @@ getinum(const char *s, int *i)
 		return 0;
 	extern double   _grMissingValue;
 	double          d;
-	char *            ptr = NULL;
+	char           *ptr = NULL;
 	d = _grMissingValue;
 	*i = (int) strtod(s, &ptr);
 	if (*ptr == '\0') {
@@ -649,6 +965,22 @@ getinum(const char *s, int *i)
 				return true;
 			}
 		}
+	} else if (is_syn(s)) {
+		std::string syn_value;
+		bool exists = get_syn(s, syn_value);
+		//printf("DEBUG %s:%d '%s' exists= %d  value [%s]\n",__FILE__,__LINE__,s,exists,syn_value.c_str());
+		if (exists) {
+			const char* vptr = syn_value.c_str();
+			ptr = NULL;	// reset this
+			*i = int(strtod(vptr, &ptr));
+			if (*ptr == '\0') {
+				return true;
+			} else {
+				return false;
+			}
+		} else {
+			return false;
+		}
 	} else if (!strcmp(s, "-NaN") 
 		   || !strcmp(s, "NaN") 
 		   || !strcmp(s, "Inf")
@@ -672,10 +1004,17 @@ getinum(const char *s, int *i)
 	// whether the user is interested in errors.
 	*i = (int) _grMissingValue;
 	if (!_ignore_error) {
-		err("Cannot interpret `\\",
-		    s,
-		    "' as a number.",
-		    "\\");
+		if (isalpha(s[1])) {
+			err("Cannot interpret `\\",
+			    s,
+			    "' as a number.\n       Did you forget the final \".\" in a variable name?",
+			    "\\");
+		} else {
+			err("Cannot interpret `\\",
+			    s,
+			    "' as a number.",
+			    "\\");
+		}
 	}
 	return false;
 }
@@ -685,22 +1024,25 @@ bool
 getdnum(const char *s, double *d)
 {
 	if (*s == '\0')
-		return 0;
-	extern double   _grMissingValue;
-	char *            ptr = NULL;
-	*d = strtod(s, &ptr);
-	if (*ptr == '\0') {
-		// Normal number; check for infinity/not-a-number
-#if defined(HAVE_ISNAN) && defined(HAVE_ISINF)
-#if !defined(__MACHTEN__)
-		if (isinf(*d) || isnan(*d))
-			*d = (double) _grMissingValue;
-#endif
+		return false;
+	if (get_normal_number(s, d)) {
+#ifdef DEBUG_UNDERSCORE
+		printf("%s:%d normal number '%s' = %e\n", __FILE__,__LINE__,s, *d);
 #endif
 		return true;
 	}
+	if (get_number_with_underscores(s, d)) {
+#ifdef DEBUG_UNDERSCORE
+		printf("%s:%d decoded underline '%s' to be %e\n", __FILE__,__LINE__,s, *d);
+#endif
+		return true;
+	}
+#ifdef DEBUG_UNDERSCORE
+	printf("%s:%d OK, not any kinda number '%s'\n", __FILE__,__LINE__,s);
+#endif
 	// Cannot read as a normal number.  Check to see if it's a variable or
 	// NaN/Inf.
+	extern double _grMissingValue;
 	if (is_var(s)) {
 		if (get_var(s, d)) {
 			return true;
@@ -713,6 +1055,25 @@ getdnum(const char *s, double *d)
 				return true;
 			}
 		}
+#if 1				// vsn 2.6.0 [2001-feb-18]
+	} else if (is_syn(s)) {
+		std::string syn_value;
+		bool exists = get_syn(s, syn_value);
+		//printf("DEBUG %s:%d '%s' exists= %d  value [%s]\n",__FILE__,__LINE__,s,exists,syn_value.c_str());
+		if (exists) {
+			const char* vptr = syn_value.c_str();
+			char *ptr;
+			ptr = NULL;	// reset this
+			*d = strtod(vptr, &ptr);
+			if (*ptr == '\0') {
+				return true;
+			} else {
+				return false;
+			}
+		} else {
+			return false;
+		}
+#endif
 	} else if (!strcmp(s, "-NaN") || !strcmp(s, "NaN") || !strcmp(s, "-Inf") || !strcmp(s, "Inf")) {
 		*d = _grMissingValue;
 		return true;
@@ -722,17 +1083,26 @@ getdnum(const char *s, double *d)
 		int loc;
 		loc=tmp_string.find('d'); if (-1!=loc) tmp_string.replace(loc,1,"e");
 		loc=tmp_string.find('D'); if (-1!=loc) tmp_string.replace(loc,1,"e");
+		char *ptr;
 		*d = (double)strtod(tmp_string.c_str(), &ptr);
 		if (*ptr == '\0')
 			return true;
 	}
 	// It's a mystery.  report error, fatal or nonfatal depending on
 	// whether the user is interested in errors.
+	*d = (double) _grMissingValue;
 	if (!_ignore_error) {
-		err("Cannot interpret `\\",
-		    s,
-		    "' as a number.",
-		    "\\");
+		if (isalpha(s[1])) {
+			err("Cannot interpret `\\",
+			    s,
+			    "' as a number.\n       Did you forget the final \".\" in a variable name?",
+			    "\\");
+		} else {
+			err("Cannot interpret `\\",
+			    s,
+			    "' as a number.",
+			    "\\");
+		}
 	}
 	return false;
 }
@@ -750,9 +1120,9 @@ fatal_err(const char *string,...)
 	int             len;
 	char *            p;
 	va_list         ap;
-	if (!_error_in_cmd && _beep)
+	if (!_error_in_cmd && _gri_beep)
 		gr_textput("\007");
-	/* Have standard args */
+
 	if (string != NULL) {
 		va_start(ap, string);
 		strcpy(msg, string);
@@ -777,42 +1147,41 @@ fatal_err(const char *string,...)
 	//
 	// Print file:line so emacs mode can locate error
 	//
-	if (((unsigned) superuser()) & FLAG_AUT1) {
+	if (superuser() & FLAG_AUT1) {
 		// Remove quotes around source indicator
 		extern char source_indicator[];
-		sprintf(msg, " Error detected at %s\n", source_indicator);
+		sprintf(msg, " Error at %s\n", source_indicator);
 	} else {
 		if (block_level() > 0) {
 			if (block_source_file() != NULL) {
-				printf("DEBUG utility.cc:fatal_err(): block_source_line()=%d\n",block_source_line());
+				if (superuser() & FLAG_AUT1)printf("DEBUG utility.cc:fatal_err(): block_source_line()=%d\n",block_source_line());
 				sprintf(msg,
-					" Error detected at %s:%d\n",
+					" Error at %s:%d\n",
 					block_source_file(),
-					block_source_line() + block_offset_line() - 1);
-			} else {
-				strcat(msg, "");
+					block_source_line() - 1);
+				gr_textput(msg);
 			}
 		} else {
-			if (what_file() != NULL)
+			if (what_file() != NULL) {
 				sprintf(msg,
-					" Error detected at %s:%d\n",
+					" Error at %s:%d\n",
 					what_file(),
 					what_line());
-			else
-				strcat(msg, "");
+				gr_textput(msg);
+			}
 		}
 	}
-	gr_textput(msg);
-	if (_error_action == 1) {
+	if (_error_action == 1)
 		gri_abort();
-	} else {
+	else
 		gri_exit(1);
-	}
 }
 
-char           *
+const char *
 what_file()
 {
+	if (_cmdFILE.size() == 0)
+		return NULL;
 	if (block_level() > 0) {
 		if (_cmd_being_done < 1) {
 			return NULL;
@@ -853,7 +1222,7 @@ err(const char *string,...)
 	int             len;
 	char *            p;
 	va_list         ap;
-	if (!_error_in_cmd && _beep)
+	if (!_error_in_cmd && _gri_beep)
 		gr_textput("\007");
 	if (string != NULL) {
 		va_start(ap, string);
@@ -886,13 +1255,13 @@ void
 warning(const char *s,...)
 {
 	static          unsigned int msg_last_copies = 0;
-	static string   msg_last;	// must be static
-	static string msg;		// static only for efficiency if called a lot
+	static std::string   msg_last;	// must be static
+	static std::string msg;		// static only for efficiency if called a lot
 	bool            several = false;
 	int             len;
 	char           *p = NULL;
 	va_list         ap;
-	if (!_error_in_cmd && _beep)
+	if (!_error_in_cmd && _gri_beep)
 		gr_textput("\007");
 	// Check for final wrap-up command
 	if (!strcmp(s, "\\\\")) {
@@ -952,29 +1321,59 @@ warning(const char *s,...)
  * ExtractQuote() -- extract quote `sout' from string `s'. NOTE: You must
  * ensure sout is as long as s. RETURN VALUE: character position at end of
  * extracted quote, if all was OK.  Otherwise 0.
+ *
+ * Accepts both quoted strings '... "hi" ...' and unquoted strings '... \"hi\" ...'
+ * but not combinations.
  */
+// RETURN VALUE:
+//      0  if no quoted thing found
+//     -1  if missing final-quote
+//     >0  OK; value 'i' is such that s[i] is just after the final quote
 int
-ExtractQuote(char *sout, const char *s)
+ExtractQuote(const char *s, std::string& sout)
 {
-	int             i = 0, last_char = 0;
-	while (*(s + i) != '"') {
-		if (*(s + i) == '\0') {
-			*sout = '\0';
-			return 0;
+	//printf("DEBUG %s:%d input string is '%s'\n",__FILE__,__LINE__,s);
+	int i = 0;
+	bool slash_quoted = false;
+	// Skip along to first quote ...
+	while (s[i] != '"') {
+		if (s[i] == '\0')
+			return 0; // never found any quoted items
+		i++;
+	}
+	if (i > 0 && s[i - 1] == '\\')
+		slash_quoted = true;
+
+	i++;			// skip the quote
+	// ... then copy along until find first un-escaped quote ...
+	while (s[i] != '\0') {
+		if (s[i] == '"') {
+			if (i > 0 && s[i - 1] == '\\') {
+				if (!slash_quoted) {
+					sout += '"';
+				} else {
+					sout.STRINGERASE(sout.size() - 1, 1); // trim it
+					//printf("DEBUG %s:%d RETURNING-A %d.  Extracted quote length %d as \n<%s>\n", __FILE__,__LINE__,i+1,sout.size(),sout.c_str());
+					return i + 1;
+				}
+			} else {
+				//printf("DEBUG %s:%d RETURNING-B %d\n<%s>\n\n",__FILE__,__LINE__,i+1,sout.c_str());
+				return i + 1;
+			}
+		} else {
+			sout += s[i];
 		}
 		i++;
 	}
-	strcpy(sout, s + i + 1);
-	last_char = i;
-	i = 0;
-	while (*(sout + i) != '\0') {
-		if (*(sout + i) == '"' && *(sout + i - 1) != '\\') {
-			*(sout + i) = '\0';
-			return (2 + last_char + i);
-		}
-		i++;
+	if (s[i] == '\0') {
+		//printf("DEBUG %s:%d RETURNING-C -1.  Extracted quote length %d as \n<%s>\n", __FILE__,__LINE__,sout.size(),sout.c_str());
+		return -1;
 	}
-	return (2 + last_char + i);
+
+	// ... and return an index so the parser can do more
+	// work on 's' past the quote
+	//printf("DEBUG %s:%d RETURNING %d.  Extracted quote length %d as \n<%s>\n", __FILE__,__LINE__,i,sout.size(),sout.c_str());
+	return i;
 }
 
 // Make all trailing blanks, tabs, etc, into null chars
@@ -992,7 +1391,7 @@ remove_trailing_blanks(char *s)
 void
 beep_terminal()
 {
-	if (_beep)
+	if (_gri_beep)
 		fprintf(stderr, "\007");
 }
 
@@ -1169,7 +1568,7 @@ batch()
 		return false;
 }
 
-int
+unsigned int
 superuser()
 {
 	return _griState.superuser();
@@ -1182,23 +1581,24 @@ more_file_to_terminal(const char *filename)
 	char            sys_cmd[100];
 #if defined(VMS)		// vax-vms machine
 	sprintf(sys_cmd, "TYPE %s/PAGE", filename);
-	system(sys_cmd);
 #elif defined(MSDOS)		// ibm-style msdos machine
 	GriString fn(filename);
 	fn.convert_slash_to_MSDOS();
 	sprintf(sys_cmd, "COMMAND.COM MORE < %s", fn.getValue());
-	system(sys_cmd);
 #else				// a neutral machine
-	char *            egetenv(const char *);
-	char *            pager = egetenv("PAGER");
-	if (pager) {
+	char *egetenv(const char *);
+	char *pager = egetenv("PAGER");
+	if (*pager != '\0')
 		sprintf(sys_cmd, "%s %s", pager, filename);
-		system(sys_cmd);
-	} else {
+	else
 		sprintf(sys_cmd, "more %s", filename);
-		system(sys_cmd);
-	}
 #endif
+	if (((unsigned) superuser()) & FLAG_SYS) {
+		ShowStr("\nSending the following command to the operating system:\n");
+		ShowStr(sys_cmd);
+		ShowStr("\n");
+	}
+	call_the_OS(sys_cmd, __FILE__, __LINE__);
 }
 
 bool
@@ -1214,8 +1614,6 @@ demonstrate_command_usage()
 	return true;
 }
 
-bool            find_target(const char *pattern, int *pindex, int plen, char *target, int *tlen, int *star, int *plus);
-
 // Regular expression search. This is limited; presently can match:
 
 // (1) characters; (2) alternative characters given in square brackets; (3)
@@ -1228,22 +1626,21 @@ bool            find_target(const char *pattern, int *pindex, int plen, char *ta
 bool
 re_compare(const char *s, const char *pattern)
 {
+	bool find_target(const char *pattern, int *pindex, int plen, std::string& target, int *star, int *plus);
 	int             slen = strlen(s);
 	int             plen = strlen(pattern); 
-	int             tlen;	// for target
 	int             sindex = 0;
 	int             pindex = 0;
-	int             tindex = 0;
 	int             star;	// is subpattern followed by '*'?
 	int             plus;	// is subpattern followed by '+'?
 	bool            need_new_target = true;
 	int             matches = 0;
 	// Search through pattern
-	GriString target(strlen(pattern));
+	std::string target("");
 	while (sindex < slen) {
 		bool            this_matches = false;
 		if (need_new_target) {
-			if (!find_target(pattern, &pindex, plen, target.getValue(), &tlen, &star, &plus)) {
+			if (!find_target(pattern, &pindex, plen, target, &star, &plus)) {
 #ifdef DEBUG_RE
 				printf("ran out of pattern\n");
 #endif
@@ -1254,9 +1651,9 @@ re_compare(const char *s, const char *pattern)
 		// Now see if source string matches
 #ifdef DEBUG_RE
 		printf("source[%d] = `%c'  ", sindex, s[sindex]);
-		show_pattern(target.getValue(), tlen, star, plus);
+		show_pattern(target.c_str(), target.size(), star, plus);
 #endif
-		for (tindex = 0; tindex < tlen; tindex++) {
+		for (unsigned int tindex = 0; tindex < target.size(); tindex++) {
 			if (s[sindex] == target[tindex] || target[tindex] == '.') {
 				this_matches = true;
 				matches++;
@@ -1292,10 +1689,10 @@ re_compare(const char *s, const char *pattern)
 	}
 	// Some pattern left.  See if all remaining targets are '*' types; then
 	// have a match.
-	while (find_target(pattern, &pindex, plen, target.getValue(), &tlen, &star, &plus)) {
+	while (find_target(pattern, &pindex, plen, target, &star, &plus)) {
 		if (!star) {
 #ifdef DEBUG_RE
-			printf(" ... still some non-* target left: `%s'\n", target.getValue());
+			printf(" ... still some non-* target left: `%s'\n", target.c_str());
 #endif
 			return false;
 		}
@@ -1305,12 +1702,13 @@ re_compare(const char *s, const char *pattern)
 }
 
 bool
-find_target(const char *pattern, int *pindex, int plen, char *target, int *tlen, int *star, int *plus)
+find_target(const char *pattern, int *pindex, int plen, std::string& target, int *star, int *plus)
 {
+	target = "";
 	/*
 	 * Determine present target, leaving *pindex pointing at next part of
 	 * pattern.
-     */
+	 */
 	*star = *plus = 0;
 	if (*pindex >= plen)
 		return false;
@@ -1321,18 +1719,14 @@ find_target(const char *pattern, int *pindex, int plen, char *target, int *tlen,
 		 */
 		switch (pattern[*pindex + 1]) {
 		case '\\':
-			target[0] = '\\';
-			target[1] = '\0';
-			*tlen = 1;
+			target += '\\';
 			(*pindex)++;	/* the backslash */
 			(*pindex)++;	/* the '\\' */
 			break;
 		case 's':		/* whitespace as in perl */
-			target[0] = ' ';	// blank
-			target[1] = '\t';	// tab
-			target[2] = '\n';	// newline
-			target[3] = '\0';
-			*tlen = 3;
+			target += ' ';
+			target += '\t';
+			target += '\n';
 			(*pindex)++;	/* the backslash */
 			(*pindex)++;	/* the 's' */
 			break;
@@ -1346,20 +1740,15 @@ find_target(const char *pattern, int *pindex, int plen, char *target, int *tlen,
 		 * List of alternatives.
 		 */
 		(*pindex)++;		/* skip the '[' */
-		*tlen = 0;
-		while (pattern[*pindex] != ']' && pattern[*pindex] != '\0') {
-			target[(*tlen)++] = pattern[(*pindex)++];
-		}
+		while (pattern[*pindex] != ']' && pattern[*pindex] != '\0')
+			target += pattern[(*pindex)++];
 		(*pindex)++;		/* skip the ']' */
-		target[*tlen] = '\0';
 		break;
 	default:
 		/*
-		 * Single character.
+		 * A single character.
 		 */
-		target[0] = pattern[(*pindex)++];
-		target[1] = '\0';
-		*tlen = 1;
+		target += pattern[(*pindex)++];
 	}
 	if (pattern[*pindex] == '*') {
 		*star = 1;
@@ -1707,3 +2096,152 @@ lapse_rate(double S, double t, double p)
     
 	return Gamma;
 }
+
+// Get name for temporary file (hide details of libraries here)
+char*
+tmp_file_name()
+{
+	char *rval;
+#if defined(HAVE_TEMPNAM)
+	rval = tempnam("/usr/tmp", "gri");
+	if (return_value == NULL) 
+		return NULL;
+	return rval;
+#else
+#if defined(HAVE_TMPNAM)
+	rval = tmpnam(NULL);
+	if (rval == NULL)
+		return NULL;
+	return rval;
+#else
+	return GRI_TMP_FILE;
+#endif
+#endif
+}
+
+int
+call_the_OS(const char* cmd, const char* calling_filename, int calling_line) 
+{
+	std::string c(cmd);
+	clean_blanks_quotes(c);
+	c.append("\n");
+	if (((unsigned) superuser()) & FLAG_SYS) {
+		printf("Sending the following command to the operating system [ref: %s:%d]:\n%s\n", 
+		       calling_filename, calling_line, c.c_str());
+	}
+	int status = system(c.c_str());
+	PUT_VAR("..exit_status..", double(status));
+	return status;
+}
+
+void
+clean_blanks_quotes(std::string& c)
+{
+	// Trim any blanks at the start and end ...
+	while (isspace(c[0]))
+		c.STRINGERASE(0,1);
+	while (c.size() > 0 && isspace(c[-1 + c.size()]))
+		c.STRINGERASE(-1 + c.size(), 1);
+
+	// ... and, if the first nonblank symbol is a quote,
+	// then remove both it and a matching trailing quote.
+	if (c[0] == '"') {
+		c.STRINGERASE(0,1);
+		if (c.size() > 0 && c[-1 + c.size()] == '"')
+			c.STRINGERASE(-1 + c.size(), 1);
+	}
+}
+
+bool
+is_even_integer(double v)
+{
+	int iv = int(v);
+	if (double(iv) != v)
+		return false;	// not even an integer
+	int iiv = 2 * (iv / 2);
+	if (iv == iiv)
+		return true;
+	return false;
+}
+
+bool
+is_odd_integer(double v)
+{
+	int iv = int(v);
+	if (double(iv) != v)
+		return false;	// not even an integer
+	int iiv = 2 * (iv / 2);
+	if (iv != iiv)
+		return true;
+	return false;
+}
+
+void
+de_reference(std::string& syn)
+{
+	//printf("%s:%d 1. de_reference (%s)...\n",__FILE__,__LINE__,syn.c_str());
+	if (syn[0] == '\\' && syn[1] == '@') {
+		std::string deref("\\");
+		deref.append(syn.substr(2, syn.size()));
+		//printf("2. deref= <%s>\n", deref.c_str());
+		std::string buf;
+		if (get_syn(deref.c_str(), buf)) {
+			syn.assign(buf);
+			//printf("3. syn= <%s>\n", syn.c_str());
+			if (syn[0] == '\\' && syn[1] == '\\')
+				syn.STRINGERASE(0, 1);
+			//printf("4. syn= <%s>\n", syn.c_str());
+		}
+	}
+	//printf("%s:%d de_reference returning MODIFIED TO <%s>\n",__FILE__,__LINE__,syn.c_str());
+}
+
+void
+un_double_slash(std::string& word)	// change leading double-backslash to single-backslash
+{
+	if (word[0] == '\\' && word[1] == '\\')
+		word.STRINGERASE(0, 1);
+}
+
+void
+
+un_double_quote(std::string& word)
+{
+	if (word[0] == '"') 
+		if (word[word.size() - 1] == '"') {
+			word.STRINGERASE(word.size() - 1, 1);
+			word.STRINGERASE(0, 1);
+		}
+}
+
+void
+fix_negative_zero(std::string& number) // change e.g. "-0" to "0", for axes
+{
+	bool is_zero = true;
+	unsigned int i = 0;
+	for (; i < number.size(); i++)
+		if (number[i] != ' ')
+			break;
+	if (i == number.size() || number[i] != '-')
+		return;
+	for (i++; i < number.size(); i++) {
+		if (number[i] != '0' && number[i] != '.') {
+			is_zero = false;
+			break;
+		}
+	}
+	if (is_zero)
+		number.STRINGERASE(0, 1);
+}
+
+// Byte swapping, from /usr/include/bits/byteswap.h on a linux box
+#define gri_bswap_constant_32(x) \
+     ((((x) & 0xff000000) >> 24) | (((x) & 0x00ff0000) >>  8) | \
+      (((x) & 0x0000ff00) <<  8) | (((x) & 0x000000ff) << 24))
+unsigned int 
+endian_swap_uint(unsigned int v)
+{
+	return gri_bswap_constant_32(v);
+}
+#undef gri_bswap_constant_32
+
